@@ -24,6 +24,71 @@ This file is constituded with MLE tools used for fitting stellar
 background.
 '''
 
+def create_background_guess_arrays (freq, psd, r=1., m=1., teff=5770., 
+                                    dnu=None, numax=None,
+                                    n_harvey=2, spectro=False, power_law=False, 
+                                    high_cut_plaw=20.) :
+  '''
+  Create guess and bound arrays for background fit.
+
+  :param freq: frequency vector
+  :type freq: ndarray
+
+  :param psd: real power vector
+  :type freq: ndarray
+
+  :param n_harvey: number of Harvey laws to use to build the background
+    model. Optional, default 2. With more than two Harvey laws, it is strongly recommended
+    to manually feed the ``guess`` parameter.  
+  :type n_harvey: int
+
+  :param r: stellar radius. Given in solar radius. Optional, default 1.
+  :type r: float
+
+  :param m: stellar mass. Given in solar mass. Optional, default 1.
+  :type m: float
+
+  :param teff: stellar effective temperature. Given in K. Optional, default 5770. 
+  :type teff: float
+
+  :param dnu: large separation of the p-mode. If given, it will superseed the scaling law guess
+    using ``r``, ``m`` and ``teff``. 
+  :type dnu: float
+
+  :param numax: maximum p-mode bump power frequency. If given, it will superseed the scaling law guess
+    using ``r``, ``m`` and ``teff``. 
+  :type numax: float
+
+  :param power_law: set to ``True`` to fit a power law on the background. Optional, default 
+    ``False``.
+  :type power_law: bool
+
+  :param spectro: if set to ``True``, make the plot with unit consistent with radial velocity, else with 
+    photometry. Automated guess will also be computed consistently with spectroscopic measurements. 
+    Optional, default ``True``.
+  :type spectro: bool
+
+  :param high_cut_plaw: high cut in frequency to compute the power law guess. Optional,
+    default 20. 
+  :type high_cut_plaw: float
+
+  :return: guess, low bounds and up bounds array
+  :rtype: tuple of arrays
+  '''
+
+  if dnu is None :
+    dnu = dnu_scale (r, m)
+  if numax is None :
+    numax = numax_scale (r, m, teff)
+
+  guess = background_guess (freq, psd, numax, dnu, m_star=m, n_harvey=n_harvey,
+                            spectro=spectro, power_law=power_law, 
+                            high_cut_plaw=high_cut_plaw) 
+  low_bounds = get_low_bounds (guess, n_harvey, freq, psd, numax)
+  up_bounds = get_up_bounds (guess, n_harvey, freq, psd, numax)
+
+  return guess, low_bounds, up_bounds
+
 def create_labels (n_harvey, frozen_param) :
   '''
   Create array of label to use when plotting MCMC.
@@ -95,7 +160,7 @@ def numax_scale (r, m, teff) :
   numax_sun = 3050. 
   teff_sun = 5770.
 
-  numax = numax_sun * np.power (r, -2) * m * np.power (teff/teff_sun, -0.5)
+  numax = numax_sun * r**-2 * m * (teff/teff_sun)**-0.5
 
   return numax
 
@@ -106,7 +171,7 @@ def dnu_scale (r, m) :
   '''
 
   dnu_sun = 135.
-  dnu = dnu_sun * np.power (m, 0.5) * np.power (r, -1.5)
+  dnu = dnu_sun * m**0.5 * r**-1.5
 
   return dnu
 
@@ -652,10 +717,10 @@ def perform_mle_background (freq, psd, n_harvey=2, r=1., m=1., teff=5770., dnu=N
 
   if dnu is None :
     dnu = dnu_scale (r, m)
-    print ('Dnu computed with scale law: {:.1f} muHz'.format (dnu)) 
+    print ('dnu computed with scaling law: {:.1f} muHz'.format (dnu)) 
   if numax is None :
     numax = numax_scale (r, m, teff)
-    print ('numax computed with scale law: {:.1f} muHz'.format (numax)) 
+    print ('numax computed with scaling law: {:.1f} muHz'.format (numax)) 
 
   lenguess = n_harvey * 3 + 6
 
@@ -819,11 +884,11 @@ def explore_distribution_background (freq, psd, n_harvey=2, r=1., m=1., teff=577
   :type fit_log: bool
 
   :param low_bounds: lower bounds to consider in the parameter space exploration. Must have the same structure
-    than guess.
+    than ``guess``.
   :type low_bounds: ndarray
 
   :param up_bounds: upper bounds to consider in the parameter space exploration. Must have the same structure
-    than guess.
+    than ``guess``.
   :type up_bounds: ndarray
 
   :param spectro: if set to ``True``, make the plot with unit consistent with radial velocity, else with 
@@ -1013,7 +1078,8 @@ def explore_distribution_background (freq, psd, n_harvey=2, r=1., m=1., teff=577
     
     #saving parameters name and normalisation information
     filemcmc_info = filemcmc[:len(filemcmc)-3] + '.dat'
-    np.savetxt (filemcmc_info, np.c_[norm], fmt='%-s') 
+    np.savetxt (filemcmc_info, np.c_[norm, labels], fmt='%-s',
+                header=make_header_utility_file ()) 
 
     if save_only_after_sampling :
       # I have deliberately created the file to signal to other process that this chain is being
