@@ -3,7 +3,7 @@ import pandas as pd
 import emcee
 import corner
 from .likelihood import log_likelihood, cond_transf
-from .fit_tools import a2z_df_to_param, a2z_to_pkb, save_a2z
+from .fit_tools import *
 from .analyse_window import sidelob_param
 from .save_chain import save_sampled_chain
 import matplotlib
@@ -279,9 +279,11 @@ def explore_distribution (result, param_type, freq, psd, back,
     #saving parameters name and normalisation information
     filename_info = filename[:len(filename)-3] + '.dat'
     if degrees is not None :
-      np.savetxt (filename_info, np.c_[param_type, norm, degrees], fmt='%-s') 
+      np.savetxt (filename_info, np.c_[param_type, norm, degrees], fmt='%-s',
+                  header=make_header_utility_file ()) 
     else :
-      np.savetxt (filename_info, np.c_[param_type, norm], fmt='%-s') 
+      np.savetxt (filename_info, np.c_[param_type, norm], fmt='%-s',
+                  header=make_header_utility_file ()) 
   else :
     backend = None
 
@@ -301,54 +303,7 @@ def explore_distribution (result, param_type, freq, psd, back,
     if save_only_after_sampling :
       save_sampled_chain (filename, sampler, ndim, nwalkers, nsteps)
 
-  return sampler, norm 
-
-def show_chain (filename, thin=100, read_info=False, save=False, discard=0,
-                fit_amp=False, bins=100) :
-  '''
-  Framework to plot chains saved as hdf5 files.
-
-  :param filename: name of the chain. The name format is the following
-  'mcmc_sampler_order_[n_order]_degrees_[n_degrees].h5' with n_degrees being '13' or '02'.
-  :type filename: str
-
-  :param bins: number of bins for each cornerplot panel. Optional, default 100.
-  :type bins: int
-
-  :return: None
-  '''
-
-  reader = emcee.backends.HDFBackend(filename, read_only=True)
-  
-  if read_info :
-    labels = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=0)
-    if fit_amp :
-      labels[labels=='height'] = 'amplitude'
-    if projected_splittings :
-      labels[labels=='split'] = 'proj_split'
-    degrees = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=2)
-    labels = np.char.add (labels, np.full (labels.size, '_', dtype=str))
-    labels = np.char.add (labels, degrees)
-    norm = np.loadtxt (filename[:len(filename)-3]+'.dat', usecols=1)
-  else :
-    labels=None
-
-  flatchain = reader.get_chain(flat=True, thin=thin, discard=discard)
-  quantiles = [0.16, 0.84]
-
-  if read_info :
-    flatchain = flatchain * norm
-    labels, degrees, flatchain = sort_chain (labels, degrees, flatchain)
-    fig = corner.corner(flatchain, labels=labels, quantiles=quantiles, show_titles=True, title_fmt='.4f', bins=bins)
-  else :
-    fig = corner.corner(flatchain, labels=labels, quantiles=quantiles, show_titles=True, title_fmt='.4f', bins=bins)
-  fig.set_size_inches(24,24)
-
-  if save :
-    plt.savefig (filename[:len(filename)-3]+'.pdf', format='pdf')
-  plt.close ()
-
-  return
+  return sampler, norm, param_type, degrees 
 
 def wrap_explore_distribution (df_a2z, freq, psd, back, low_bound_freq=1500, up_bound_freq=5000, wdw=None,
                                nsteps=1000, show_corner=False, filename=None, parallelise=False, progress=False,
@@ -412,15 +367,17 @@ def wrap_explore_distribution (df_a2z, freq, psd, back, low_bound_freq=1500, up_
   cp_result = np.copy (result)
   cp_bounds = np.copy (bounds)
 
-  sampler, norm = explore_distribution (cp_result, param_type, freq, psd, back,
+  sampler, norm, labels, degrees = explore_distribution (cp_result, param_type, freq, psd, back,
                             df_to_pass, cp_bounds, low_bound_freq=low_bound_freq, filename=filename, 
                             up_bound_freq=up_bound_freq, param_wdw=param_wdw, nsteps=nsteps, 
                             parallelise=parallelise, progress=progress, nwalkers=nwalkers, normalise=normalise,
                             degrees=degrees, instr=instr, use_sinc=use_sinc, asym_profile=asym_profile,
                             save_only_after_sampling=save_only_after_sampling, fit_amp=fit_amp, 
                             projected_splittings=projected_splittings, existing_chains=existing_chains)
-  if sampler is None :
-    return None #sampler is None mean the hdf5 file already existed and no sampling has been performed
+  if sampler is None : 
+    #sampler is None mean the hdf5 file already existed and no sampling has been performed 
+    # (due to ``ignore`` option)
+    return None 
 
   # Now exploiting results from the sampler
 
@@ -430,15 +387,13 @@ def wrap_explore_distribution (df_a2z, freq, psd, back, low_bound_freq=1500, up_
   flat_samples = sampler.get_chain(discard=discard, thin=thin, flat=True)
 
   if show_corner :
-    labels = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=0)
     if fit_amp :
       labels[labels=='height'] = 'amplitude'
     if projected_splittings :
       labels[labels=='split'] = 'proj_split'
-    degrees = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=2)
+    labels = labels.astype (str)
     labels = np.char.add (labels, np.full (labels.size, '_', dtype=str))
     labels = np.char.add (labels, degrees)
-    norm = np.loadtxt (filename[:len(filename)-3]+'.dat', usecols=1)
     quantiles = [0.16, 0.84]
 
     flat_samples = flat_samples * norm
@@ -448,6 +403,8 @@ def wrap_explore_distribution (df_a2z, freq, psd, back, low_bound_freq=1500, up_
     fig.set_size_inches(24,24)
     if filename is not None :
       plt.savefig (filename[:len(filename)-3]+'.pdf', format='pdf')
+    else :
+      plt.show ()
     plt.close ()
 
   centiles = np.percentile(flat_samples, [16, 50, 84], axis=0) * norm 
