@@ -7,9 +7,12 @@ import os
 from .fit_tools import *
 from .a2z_no_pandas import wrapper_cf_to_pkb_extended_nopandas
 
-def read_chain (filename, thin=1, discard=0) :
+def read_chain (filename, thin=1, discard=0, read_order=True,
+                chain_type='peakbagging', fit_amp=False, 
+                projected_splittings=False) :
   '''
-  Read chain and return flattened sampled chain with auxiliary informations.
+  Read a chain created by ``apollinaire`` and return flattened 
+  sampled chain with auxiliary informations.
 
   :param filename: name of the hdf5 file with the stored chain.
   :type filename: str
@@ -20,21 +23,95 @@ def read_chain (filename, thin=1, discard=0) :
   :param discard: number of iteration step to discard. Optional, default 0.
   :type discard: int
 
-  :return: tuple with flatchain, labels of param, degrees of param and order of modes.
+  :param read_order: set to ``False`` when reading a ``peakbagging`` chain created
+    with the ``strategy=global``. The ``order`` output will be ``-9999``
+    in this case.  
+  :type read_order: bool 
+
+  :param chain_type: can be ``peakbagging``, ``pattern`` or ``background``.
+    Optional, default ``peakbagging``.
+  :type chain_type: str
+
+  :param fit_amp: set to ``True`` if amplitudes were fitted instead of heights.
+  :type fit_amp: bool
+
+  :param projected_splittings: set to ``True`` if projected splittings were 
+    fitted instead of splittings.
+  :type projected_splittings: bool
+
+  :return: tuple with flatchain, param labels, degrees of param and order of modes
+    if ``chain_type=peakbagging``, flatchain and labels otherwise.
   :rtype: tuple of ndarrays
   ''' 
   reader = emcee.backends.HDFBackend(filename, read_only=True)
 
-  labels = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=0)
-  degrees = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=2)
-  norm = np.loadtxt (filename[:len(filename)-3]+'.dat', usecols=1)
+  if chain_type=='peakbagging' :
+    labels = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=0)
+    degrees = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=2)
+    norm = np.loadtxt (filename[:len(filename)-3]+'.dat', usecols=1)
 
-  order = path.basename (filename)[19:21]
-  order = int (order)
+    if fit_amp :
+      labels[labels=='height'] = 'amplitude'
+    if projected_splittings :
+      labels[labels=='split'] = 'proj_split'
+    if read_order :
+      order = path.basename (filename)[19:21]
+      order = int (order)
+    else :
+      order = -9999
 
-  flatchain = reader.get_chain(flat=True, thin=thin, discard=discard) * norm
+    flatchain = reader.get_chain(flat=True, thin=thin, discard=discard) * norm
+    return flatchain, labels, degrees, order 
 
-  return flatchain, labels, degrees, order 
+  else :
+    norm = np.loadtxt (filename[:len(filename)-3]+'.dat', usecols=0)
+    labels = np.loadtxt (filename[:len(filename)-3]+'.dat', dtype=str, usecols=1)
+    flatchain = reader.get_chain(flat=True, thin=thin, discard=discard) * norm 
+    return flatchain, labels 
+
+def cornerplot_flatchain (flatchain, labels=None, filename=None, 
+                          bins=10) : 
+  '''
+  Make a cornerplot from a flattened chain. 
+
+  :param flatchain: the flattened chain.
+  :type flatchain: ndarray
+
+  :param labels: parameters labels
+  :type labels: array-like
+
+  :param filename: name of the file where the cornerplot will 
+    be saved.
+  :type filename: str
+
+  :param bins: number of bins in the histograms.
+    Optional, default ``10``. 
+  :type bins: int
+  '''
+
+  ndim = flatchain.shape[1]
+  fig = corner.corner(flatchain, bins=bins, labels=labels, 
+                      quantiles=[0.16,0.84], show_titles=True, title_fmt='.4f')
+  fig.set_size_inches(24,24)
+  axes = np.array(fig.axes).reshape((ndim, ndim))
+  for yi in range(ndim):
+    for xi in range(yi):
+      ax = axes[yi, xi]
+      ax.tick_params (labelsize=6)
+      ax.xaxis.set_major_formatter(ticker.ScalarFormatter(useMathText=True))
+      ax.yaxis.set_major_formatter(ticker.ScalarFormatter(useMathText=True))
+      ax.ticklabel_format (scilimits=(0,0), useOffset=False)
+      if xi!=0 :
+        ax.yaxis.set_ticklabels ([])
+      if yi!=ndim-1 :
+        ax.xaxis.set_ticklabels ([])
+  if filename is not None :
+    plt.savefig (filename)
+  else :
+    plt.show ()
+  plt.close ()
+
+  return
 
 def complete_df_with_ampl (df, instr='geometric') :
 
