@@ -19,6 +19,46 @@ import corner
 import numba
 import warnings
 
+def make_header_utility_file () :
+  header = 'Utility file with normalisation parameters and parameters info'
+  return header
+
+def apodisation (dt, filename=None, show=False,
+                 logscale=False) :
+  '''
+  Compute apodisation power attenuation function
+  related to sampling and plot corresponding 
+  figure.
+  '''
+  
+  fig, ax = plt.subplots (1, 1, figsize=(12, 6))
+  if not isinstance(dt, (list, tuple, np.ndarray)) :
+    dt = [dt]
+
+  for elt in dt :
+    nyquist = 1 / (2*elt)
+    freq = np.linspace (0, nyquist, 10000, endpoint=True)
+    power = np.sinc (freq / (2*nyquist))
+    power = power**2
+ 
+    ax.plot (freq*1e6, (1-power)*100, label='dt = {:.0f} s'.format (elt))
+
+  ax.set_xlabel ('Frequency ($\mu$Hz)')
+  ax.set_ylabel ('Power attenuation (%)')
+
+  ax.legend ()
+
+  if logscale :
+    ax.set_xscale ('log')
+
+  if filename is not None :
+    plt.savefig (filename)
+  if show :
+    plt.show ()
+
+  return 
+  
+
 def a2z_to_cf (df) :
   '''
   Convert a a2z DataFrame into a centile DataFrame.
@@ -162,13 +202,36 @@ def check_bounds (param, low_bounds, up_bounds, labels=None) :
   return
 
 
-def make_cornerplot (sampler, ndim, discard, thin, labels, norm, filemcmc=None, bins=100) :
+def make_cornerplot (sampler, ndim, discard, thin, labels, 
+                     norm=None, filemcmc=None, bins=100) :
   '''
-   Wrapper to make cornerplots.
+  Wrapper to make cornerplots.
+
+  :param sampler: ``emcee`` sampler.
+  :type sampler: Ensemble Sampler
+
+  :param ndim: number of sampled parameter.
+  :type ndim: int
+
+  :param discard: number of steps to discard in the sample 
+    when computing the flat chain.
+  :type discard: int
+
+  :param thin: chain thinning parameter.
+  :type thin: int
+
+  :param labels: parameters labels.
+  :type labels: array-like
+
+  :param norm: normalisation factors used for the parameters. 
+    Optional, default ``None``. 
+  :type norm: ndarray.
   '''
 
   sample_to_plot = sampler.get_chain(discard=discard, thin=thin, flat=True)
-  fig = corner.corner(sample_to_plot*norm, bins=bins, labels=labels, quantiles=[0.16,0.84], show_titles=True, title_fmt='.4f')
+  if norm is not None :
+    sample_to_plot = sample_to_plot*norm
+  fig = corner.corner(sample_to_plot, bins=bins, labels=labels, quantiles=[0.16,0.84], show_titles=True, title_fmt='.4f')
   fig.set_size_inches(24,24)
   axes = np.array(fig.axes).reshape((ndim, ndim))
   for yi in range(ndim):
@@ -374,166 +437,9 @@ def a2z_to_pkb (df_a2z, nopandas=True) :
   
   if nopandas :
     return wrapper_a2z_to_pkb_nopandas (df_a2z)
-
-  modes = df_a2z.loc[df_a2z[2]=='freq'].copy ()
-  height = df_a2z.loc[df_a2z[2]=='height'].copy ()
-  width = df_a2z.loc[df_a2z[2]=='width'].copy ()
-  amp_l = df_a2z.loc[df_a2z[2]=='amp_l'].copy ()
-  asym = df_a2z.loc[df_a2z[2]=='asym'].copy ()
-  n_elt = modes.index.size
-
-  #converting columns 
-  modes.loc[:,0] = modes.loc[:,0].map (np.int_)
-  modes.loc[:,1] = modes.loc[:,1].map (np.int_)
-  height.loc[:,0] = height.loc[:,0].map (np.int_)
-  width.loc[:,0] = width.loc[:,0].map (np.int_)
-  amp_l.loc[:,1] = amp_l.loc[:,1].map (np.int_)
-  asym.loc[:,0] = asym.loc[:,0].map (np.int_)
-
-  # computing height and width with a line for each mode
-  # ------------------------------------------------------------------------
-  aux_height = modes[[0,1]].copy () 
-  aux_width = modes[[0,1]].copy ()
-  amp_l = amp_l.rename (columns={4:'ratio'})
-  if (height[1]=='a').any () : #height are fitted order by order
-    # 'correcting' order in order to have the right height and width value when performing the
-    # join step.
-    aux_height.loc[aux_height[1]>1, 0] = aux_height.loc[aux_height[1]>1, 0] + 1
-    aux_height = aux_height.set_index(aux_height[0]).join (height[[4,5]].set_index(height[0]))
-    aux_height = aux_height.set_index (aux_height[1]).join (amp_l[['ratio']].set_index(amp_l[1]))
-    aux_height.loc[:,4] = aux_height[4].to_numpy() * aux_height['ratio'].to_numpy ()
-    aux_height.loc[:,5] = aux_height[5].to_numpy() * aux_height['ratio'].to_numpy ()
-    aux_height = aux_height.reset_index (drop=True)
-    aux_height.loc[aux_height[1]>1, 0] = aux_height.loc[aux_height[1]>1, 0] - 1
-    aux_height = aux_height.rename (columns={4:'height', 5:'height_error'})
-  else : #case where heights are fitted degree to degree
-    height.loc[:,1] = height.loc[:,1].map (np.int_)
-    height = height.set_index([0,1])
-    aux_height = aux_height.set_index([0,1]).join (height[[4,5]])
-    aux_height = aux_height.reset_index ()
-    aux_height = aux_height.rename (columns={4:'height', 5:'height_error'})
-  if (width[1]=='a').any () : #width are fitted order to order
-    # 'correcting' order in order to have the right height and width value when performing the
-    # join step.
-    aux_width.loc[aux_width[1]>1, 0] = aux_width.loc[aux_width[1]>1, 0] + 1
-    aux_width = aux_width.set_index(aux_width[0]).join (width[[4,5]].set_index(width[0]))
-    aux_width = aux_width.reset_index (drop=True)
-    aux_width.loc[aux_width[1]>1, 0] = aux_width.loc[aux_width[1]>1, 0] - 1
-    aux_width = aux_width.rename (columns={4:'width', 5:'width_error'})
-  else : #case where widths are fitted degree to degree
-    width.loc[:,1] = width.loc[:,1].map (np.int_)
-    width = width.set_index([0,1])
-    aux_width = aux_width.set_index([0,1]).join (width[[4,5]])
-    aux_width = aux_width.reset_index ()
-    aux_width = aux_width.rename (columns={4:'width', 5:'width_error'})
-
-  # extracting angle, angle_error, split and split_error
-  glob_split = 0.
-  glob_split_error = 0.
-
-  cond_glob_split = (df_a2z[2]=='split')&(df_a2z[0]=='a')  #check if a global split value exists in the dataframe
-  if df_a2z.loc[cond_glob_split].index.size == 1 :
-    glob_split = df_a2z.loc[cond_glob_split, 4].values[0]
-    glob_split_error = df_a2z.loc[cond_glob_split, 5].values[0]
-
-  cond_split = (df_a2z.loc[df_a2z[2]=='split'].index.size > 1) 
-  if cond_split :
-    split = df_a2z.loc[(df_a2z[2]=='split')&(df_a2z[0]!='a')].copy ()
-    split.loc[:,0] = split.loc[:,0].map (np.int_)
-    aux_split = modes[[0,1]].copy ()
-    if (split[1]=='a').any () : #splitting are fitted order to order
-      aux_split.loc[aux_split[1]>1, 0] = aux_split[aux_split[1]>1, 0] + 1
-      aux_split = aux_split.set_index(aux_split[0]).join (split[[4,5]].set_index(split[0]))
-      aux_split = aux_split.reset_index (drop=True)
-      aux_split.loc[aux_split[1]>1, 0] = aux_split.loc[aux_split[1]>1, 0] - 1
-      aux_split = aux_split.rename (columns={4:'split', 5:'split_error'})
-    else : #splitting are fitted degree to degree
-      split.loc[:,1] = split.loc[:,1].map (np.int_)
-      split = split.set_index([0, 1])
-      aux_split = aux_split.set_index([0, 1]).join (split[[4,5]])
-      aux_split = aux_split.reset_index ()
-      aux_split.loc[aux_split[1]==0, 4] = 0   #set splitting for l=0 to 0 instead of NaN
-      aux_split.loc[aux_split[1]==0, 5] = 0
-      aux_split = aux_split.rename (columns={4:'split', 5:'split_error'})
-
-  #asymetry extraction
-  if not asym.empty : 
-    if (asym[1]=='a').any () : #asym are fitted order to order
-      aux_asym = modes[[0,1]].copy ()
-      aux_asym.loc[aux_asym[1]>1, 0] = aux_asym[0] + 1
-      aux_asym = aux_asym.set_index(aux_asym[0]).join (asym[[4,5]].set_index(asym[0]))
-      aux_asym = aux_asym.reset_index (drop=True)
-      aux_asym.loc[aux_asym[1]>1, 0] = aux_asym[0].loc[aux_asym[1]>1] - 1
-      aux_asym = aux_asym.rename (columns={4:'asym', 5:'asym_error'})
-    else : #asym are fitted degree to degree
-      asym.loc[:,1] = asym.loc[:,1].map (np.int_)
-      aux_asym = modes[[0,1]].copy ()
-      asym  = asym.set_index([0,1])
-      aux_asym = aux_asym.set_index([0, 1]).join (asym[[4,5]])
-      aux_asym = aux_asym.reset_index ()
-      aux_asym = aux_asym.rename (columns={4:'asym', 5:'asym_error'})
-
-  #joining height and width on the main frame
-  modes = modes.set_index ([0,1])
-  aux_height = aux_height.set_index ([0,1])
-  aux_width = aux_width.set_index ([0,1])
-  modes = modes.join (aux_height)
-  modes = modes.join (aux_width)
-  # joining splits
-  if cond_split :
-    aux_split = aux_split.set_index ([0,1])
-    modes = modes.join (aux_split)
-  #joining asymetries
-  if not asym.empty :
-    aux_asym = aux_asym.set_index ([0,1])
-    modes = modes.join (aux_asym)
-  modes = modes.reset_index ()
-
-  try :
-    angle = df_a2z.loc[df_a2z[2]=='angle', 4].values[0]
-    angle_error = df_a2z.loc[df_a2z[2]=='angle', 5].values[0]
-  except IndexError :
-    angle = 90.
-    angle_error = 0.
-  # ------------------------------------------------------------------------
-
-  # Filling pkb array 
-  param_pkb = np.zeros ((n_elt, 14))
-  param_pkb[:,0] = modes[0].to_numpy () #n - order
-  param_pkb[:,1] = modes[1].to_numpy () #l - degree
-  param_pkb[:,2] = modes[4].to_numpy () #nu - freq
-  param_pkb[:,3] = modes[5].to_numpy () #nu_error - freq
-  # height are given in ***^2/muHz
-  param_pkb[:,4] = modes['height'].to_numpy ()  
-  param_pkb[:,5] = modes['height_error'].to_numpy () 
-  #
-  param_pkb[:,6] = modes['width'].to_numpy () 
-  param_pkb[:,7] = modes['width_error'].to_numpy () 
-  param_pkb[:,8] = angle
-  param_pkb[:,9] = angle_error
-  mask_0 = (param_pkb[:, 1] == 0) #setting l=0 splitting to 0
-  if cond_split :
-    param_pkb[:,10] = modes['split'].to_numpy ()  
-    param_pkb[:,11] = modes['split_error'].to_numpy () 
-    param_pkb[:,10] = np.nan_to_num (param_pkb[:,10], nan=glob_split) #allow to not specify every split value
-    param_pkb[:,11] = np.nan_to_num (param_pkb[:,11], nan=glob_split_error) #in the a2z input 
-  param_pkb[mask_0,10] = 0.
-  param_pkb[mask_0,11] = 0.
-  if not asym.empty :
-    param_pkb[:,12] = modes['asym'].to_numpy ()
-    param_pkb[:,13] = modes['asym_error'].to_numpy ()
-    param_pkb[:,12] = np.nan_to_num (param_pkb[:,12]) #allow not to specify every asym value in
-    param_pkb[:,13] = np.nan_to_num (param_pkb[:,13]) #the a2z input
-
-  if (~df_a2z.loc[df_a2z[1]=='4'].empty) | (~df_a2z.loc[df_a2z[1]=='5'].empty) :
-    if (df_a2z.loc[(df_a2z[1]=='0')&(df_a2z[6]==0)].empty) :
-      l1_as_ref = True
-    else :
-      l1_as_ref = False    
-    param_pkb = complete_pkb (param_pkb, l1_as_ref)
-
-  return param_pkb
-
+  else :
+    raise Exception ('nopandas=False is deprecated and cannot be used anymore.')
+    
 def pkb_to_latex (pkb) :
   '''
   Convert a pkb array into a tabular latex string
@@ -569,9 +475,9 @@ def input_to_pkb (param_to_fit, df_info_modes, df_global) :
   return param_pkb
 
 
-def smooth (vector, smoothing) :
+def smooth (vector, smoothing, win_type='triang') :
   '''
-  Smooth routines. Uses boxcar smoothing. 
+  Smooth routines. Uses triangle smoothing by default
 
   :param vector: vector to smooth.
   :type vector: ndarray
@@ -579,6 +485,9 @@ def smooth (vector, smoothing) :
   :param smoothing: size of the rolling window used for the smooth.
   :type smoothing: int
 
+  :param win_type: see ``scipy.signal.windows``. Optional, default ``triang``.
+  :type win_type: str
+
   :return: smoothed vector
   :rtype: ndarray
   '''
@@ -608,9 +517,9 @@ def compute_model (freq, param_pkb, param_wdw=None, correct_width=1., instr='kep
                    projected_splittings=False) :
 
   '''
-  Compute the model to compare with observed PSD.
+  Compute a p-mode model from a given set of parameters.
 
-  :param freq: vector of frequency
+  :param freq: frequency vector.
   :type freq: ndarray
 
   :param param_pkb: parameters contained in the pkb files.
