@@ -12,6 +12,7 @@ from .modified_optimize import _minimize_powell
 from .regressor import gp_predict, get_min_max_teff
 from pathos.multiprocessing import ProcessPool
 from .save_chain import save_sampled_chain
+from .background import numax_scale, dnu_scale
 import numdifftools as nd
 import emcee
 import corner
@@ -26,6 +27,93 @@ initial guess to give to the MLE/MCMC peakbagging pipeline for individual
 mode fitting.
 '''
 
+def compute_model_from_pattern (freq, pattern, back, n_order=3,
+                                amp_l=[1., 1.5, 0.7, 2]) :
+  '''
+  Compute a p-mode model from global pattern parameters.
+
+  :param freq: frequency vector.
+  :type freq: ndarray
+
+  :param pattern: pattern parameters. Pattern parameters have to be given in
+    the following order: ``eps``, ``alpha``, ``Dnu``, ``numax``,  ``Hmax``,
+    ``Wenv``, ``w``, ``d02``, ``b02``, ``d01``, ``b01``, ``d13``, ``b03``. A first
+    version of the guess and bounds array can be created through the
+    ``create_pattern_guess_arrays``.
+  :type pattern: ndarray
+
+  :param back: background vector to use for the model. 
+  :type back: ndarray
+
+  :param n_order: number of orders to consider around ``numax`` to build
+    the model. Optional, default ``3``.
+  :type n_order: int
+
+  :return: computed model
+  :rtype: ndarray
+  '''
+
+  dnu = pattern[2]
+  numax = pattern[3]
+  orders = list_order_to_fit (numax, dnu, n_order=n_order)
+  pkb = pattern_to_pkb (*pattern, orders=orders, amp_l=amp_l)
+  model = compute_model (freq, pkb)
+  model = model + back
+
+  return model
+
+def create_pattern_guess_arrays (Hmax, Wenv, r=1., m=1., teff=5770., 
+                                 dnu=None, numax=None,
+                                 mcmc=True) :
+  '''
+  Create guess and bound arrays for pattern fit.
+
+  :param Hmax: height at numax.
+  :type Hmax: float
+
+  :param Wenv: width of the p-mode gaussian.
+  :type Wenv: float
+
+  :param r: stellar radius. Given in solar radius. Optional, default 1.
+  :type r: float
+
+  :param m: stellar mass. Given in solar mass. Optional, default 1.
+  :type m: float
+
+  :param teff: stellar effective temperature. Given in K. Optional, default 5770. 
+  :type teff: float
+
+  :param dnu: large separation of the p-mode. If given, it will superseed the scaling law guess
+    using ``r``, ``m`` and ``teff``. 
+  :type dnu: float
+
+  :param numax: maximum p-mode bump power frequency. If given, it will superseed the scaling law guess
+    using ``r``, ``m`` and ``teff``. 
+  :type numax: float
+ 
+  :param mcmc: if set to ``True``, will compute the guess values used for the MCMC exploration.
+    Otherwise, will compute the guess values for the MLE optimisation. Optional, default ``True``.
+  :type mcmc: bool
+
+  :return: guess, low bounds and up bounds array
+  :rtype: tuple of arrays
+  '''
+  if dnu is None :
+    dnu = dnu_scale (r, m)
+  if numax is None :
+    numax = numax_scale (r, m, teff)
+
+  if mcmc==True :
+    guess, bounds = first_guess_mcmc (dnu, numax, teff, Hmax, Wenv, 
+                                      mass=m, return_bounds=True)
+  else :
+    guess, bounds = first_guess_mle (dnu, numax, teff, Hmax, Wenv, 
+                                      mass=m, return_bounds=True)
+
+  low_bounds = bounds[:,0]
+  up_bounds = bounds[:,1]
+  return guess, low_bounds, up_bounds
+
 def first_guess_mle (Dnu, numax, Teff, Hmax, Wenv, mass=None, return_bounds=False,
                      use_gp=True) :
   '''
@@ -315,7 +403,7 @@ def hide_modes (freq, psd, pkb, back, l=1, n_width=4,
 
   :param remove_left: if set to ``True``, remove a region Dnu/2 wide
     before the first mode given in the pkb. Optional, default ``True``. 
-  :type remove_left" bool
+  :type remove_left: bool
 
   :param remove_zero_bin: if set to ``True``, bins with zero values will be removed from
     returned vector. Optional, default ``True``.
@@ -358,7 +446,8 @@ def hide_modes (freq, psd, pkb, back, l=1, n_width=4,
 
 def remove_pattern (freq, psd, back, pkb, l=[0,2]) :
   '''
-  Remove 
+  Remove a given pattern from the PSD by dividing the
+  PSD by the computed model.  
 
   :param freq: frequency vector
   :type freq: ndarray
@@ -490,7 +579,7 @@ def override_width (w, param, bounds, mcmc=True) :
 
 def pattern_to_a2z (eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, b02=0., d01=None, b01=0., 
                     d13=None, b03=0., split=0, angle=90, orders=None, splitting_order=False,
-                    angle_order=False, common_width=False) :
+                    angle_order=False, common_width=False, amp_l=[1., 1.5, 0.7, 0.2]) :
   '''
   Compute param_a2z that will be used to feed a2z_to_pkb function and then compute_model
   function.
@@ -639,10 +728,10 @@ def pattern_to_a2z (eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, b02=0., d01=None
 
   # creating global part of the dataframe
   index_n = np.array (['a', 'a', 'a', 'a', 'a', 'a']) 
-  index_l = np.array (['a', 'a', '3', '1', '2', '0']) 
+  index_l = np.array (['a', 'a', '0', '1', '2', '3']) 
   param = np.array (['split', 'angle', 'amp_l', 'amp_l', 'amp_l', 'amp_l'])
   extent = np.array (['global', 'global', 'global', 'global', 'global', 'global'])
-  value = np.array ([split, angle, 0.2, 1.5, 0.7, 1.])
+  value = np.array ([split, angle, *amp_l])
   err = np.zeros (6)
   fixed = np.zeros (6)
   l_bound = np.zeros (6)
@@ -664,13 +753,13 @@ def pattern_to_a2z (eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, b02=0., d01=None
 
 @numba.jit (nopython=True)
 def wrapped_pattern_to_pkb_fast (eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, b02=0, d01=None, b01=0,
-                    d13=None, b03=0, split=0, angle=90, orders=None) :
+                                 d13=None, b03=0, split=0, angle=90, orders=None, 
+                                 ratio_h=np.array ([1., 1.5, 0.7, 0.2])) :
   '''
   Directly compute a pkb file from the pattern parameters. 
   '''
 
   nmax = numax / Dnu - eps
-  ratio_h = np.array ([1., 1.5, 0.7, 0.2])
   
   degrees = np.array ([0, 2])
   if d01 != -9999 :
@@ -709,21 +798,23 @@ def wrapped_pattern_to_pkb_fast (eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, b02
   return pkb
 
 def pattern_to_pkb_fast (eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, b02=0, d01=None, b01=0,
-                    d13=None, b03=0, split=0, angle=90, orders=None) :
+                    d13=None, b03=0, split=0, angle=90, orders=None, amp_l=[1., 1.5, 0.7, 0.2]) :
   '''
   Directly compute a pkb file from the pattern parameters. 
   '''
+  amp_l = np.array (amp_l)
   if d13 is None :
     d13 = - 9999
   if d01 is None :
     d01 = - 9999
   
   pkb = wrapped_pattern_to_pkb_fast (eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, b02=b02, d01=d01, b01=b01,
-                    d13=d13, b03=b03, split=split, angle=angle, orders=orders)
+                    d13=d13, b03=b03, split=split, angle=angle, orders=orders, ratio_h=amp_l)
   return pkb
 
 def pattern_to_pkb (eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, b02=0, d01=None, b01=0,
-                    d13=None, b03=0, split=0, angle=90, orders=None, fast=True) :
+                    d13=None, b03=0, split=0, angle=90, orders=None, fast=True, 
+                    amp_l=[1., 1.5, 0.7, 0.2]) :
 
   '''
   Use pkb parameters to generate a pkb array.
@@ -733,11 +824,11 @@ def pattern_to_pkb (eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, b02=0, d01=None,
 
   if fast :
     param_pkb = pattern_to_pkb_fast (eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, b02=b02, d01=d01, b01=b01,
-                                     d13=d13, b03=b03, split=split, angle=angle, orders=orders)
+                                     d13=d13, b03=b03, split=split, angle=angle, orders=orders, amp_l=amp_l)
 
   else :
     df = pattern_to_a2z (eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, b02=b02, d01=d01, b01=b01,
-                         d13=d13, b03=b03, split=split, angle=angle, orders=orders)
+                         d13=d13, b03=b03, split=split, angle=angle, orders=orders, amp_l=amp_l)
     param_pkb = a2z_to_pkb (df)
 
   return param_pkb
@@ -770,7 +861,7 @@ def log_prior (param, bounds) :
   return l_prior
 
 def log_likelihood_pattern (param_pattern, freq, psd, back, orders, split=0, angle=90, param_wdw=None, 
-                            ) :
+                            amp_l=[1., 1.5, 0.7, 0.2]) :
   '''
   Compute logarithmic likelihood for a given power spectrum, with a model built only with a global
   pattern (no fit of individual mode).
@@ -795,7 +886,8 @@ def log_likelihood_pattern (param_pattern, freq, psd, back, orders, split=0, ang
   # Retrieve the width parameter from its logarithm
   param_pattern[6] = np.exp (param_pattern[6])
 
-  param_pkb = pattern_to_pkb (*param_pattern, split=split, angle=angle, orders=orders)
+  param_pkb = pattern_to_pkb (*param_pattern, split=split, angle=angle, orders=orders,
+                               amp_l=amp_l)
 
   model = compute_model (freq, param_pkb, param_wdw=param_wdw)
   model = model / back
@@ -807,7 +899,7 @@ def log_likelihood_pattern (param_pattern, freq, psd, back, orders, split=0, ang
   return log_l
 
 def log_probability_pattern (param_to_fit, freq, psd, bounds, back, orders, split=0, angle=90, param_wdw=None,
-                             norm=None) : 
+                             norm=None, amp_l=[1., 1.5, 0.7, 0.2]) : 
   '''
   Compute the positive posterior log probability (unnormalised) of the parameters to fit. 
 
@@ -849,14 +941,15 @@ def log_probability_pattern (param_to_fit, freq, psd, bounds, back, orders, spli
     return - np.inf
 
   l_likelihood = - log_likelihood_pattern (param_to_fit, freq, psd, back, orders, split, angle, param_wdw, 
-                                           )
+                                           amp_l)
 
   l_proba = l_prior + l_likelihood
 
   return l_proba
 
 def visualise (param_pattern, freq, psd, back, param_pattern_2=None, filename=None, 
-               orders=None, split=0, angle=90, snr=True, param_wdw=None, show=False) :
+               orders=None, split=0, angle=90, snr=True, param_wdw=None, show=False,
+               amp_l=[1., 1.5, 0.7, 0.2]) :
   '''
   Allow to visualise the considered global_model. May take an optional argument 
   param_pattern_2 to compare two patterns (for example guess and model fitted). 
@@ -894,7 +987,8 @@ def visualise (param_pattern, freq, psd, back, param_pattern_2=None, filename=No
   fig = plt.figure (figsize=(12,6))
   ax = fig.add_subplot (111)
 
-  param_pkb = pattern_to_pkb (*param_pattern, split=split, angle=angle, orders=orders)
+  param_pkb = pattern_to_pkb (*param_pattern, split=split, angle=angle, orders=orders,
+                              amp_l=amp_l)
   model = compute_model (freq, param_pkb, param_wdw=param_wdw)
   if snr :
     model = model / back + 1.
@@ -908,7 +1002,8 @@ def visualise (param_pattern, freq, psd, back, param_pattern_2=None, filename=No
   ax.plot (freq, psd_s, color='black')
 
   if param_pattern_2 is not None :
-    param_pkb_2 = pattern_to_pkb (*param_pattern_2, split=split, angle=angle, orders=orders)
+    param_pkb_2 = pattern_to_pkb (*param_pattern_2, split=split, angle=angle, orders=orders,
+                                  amp_l=amp_l)
     model_2 = compute_model (freq, param_pkb_2, param_wdw=param_wdw)
     if snr :
       model_2 = model_2 / back + 1.
@@ -938,9 +1033,10 @@ def visualise (param_pattern, freq, psd, back, param_pattern_2=None, filename=No
 def perform_mle_pattern (Dnu, numax, Hmax, Wenv, Teff, freq, psd, 
                          back=None, wdw=None, method=_minimize_powell,
                          n_order=3, split=0, angle=90, fit_l1=False, fit_l3=False, 
-                         mass=None, guess=None, show=True, filename=None,
-                         epsilon=None, width=None, alpha=None, d02=None,
-                         d01=None, d13=None) :
+                         mass=None, guess=None, low_bounds=None, up_bounds=None,
+                         show=True, filename=None, epsilon=None, width=None, 
+                         alpha=None, d02=None, d01=None, d13=None,
+                         amp_l=[1., 1.5, 0.7, 0.2]) :
                         
   '''
   Perform minimization of the log likelihood function according to the parameters set to vary.
@@ -979,6 +1075,14 @@ def perform_mle_pattern (Dnu, numax, Hmax, Wenv, Teff, freq, psd,
   >>>> eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, d01 (if fit_l1 is set to ``True``) <<<<
   :type guess: array-like
 
+  :param low_bounds: lower bounds to consider in the parameter space exploration. Must have the same structure
+    than ``guess``.
+  :type low_bounds: ndarray
+
+  :param up_bounds: upper bounds to consider in the parameter space exploration. Must have the same structure
+    than ``guess``.
+  :type up_bounds: ndarray
+
   :param show: if set to ``True``, will plot a summary plot of guess (blue) and fitted (red) 
   pattern.
   :type show: bool
@@ -1017,6 +1121,13 @@ def perform_mle_pattern (Dnu, numax, Hmax, Wenv, Teff, freq, psd,
 
   if guess is None :
     guess, bounds = first_guess_mle (Dnu, numax, Teff, Hmax, Wenv, mass=mass, return_bounds=True)
+  else :
+    xxx, bounds = first_guess_mle (Dnu, numax, Teff, Hmax, Wenv, mass=mass, return_bounds=True)
+
+  if low_bounds is not None :
+    bounds[:,0] = low_bounds
+  if up_bounds is not None :
+    bounds[:,1] = up_bounds
 
   if epsilon is not None :
     guess, bounds = override_epsilon (epsilon, guess, bounds, mcmc=False)
@@ -1036,7 +1147,7 @@ def perform_mle_pattern (Dnu, numax, Hmax, Wenv, Teff, freq, psd,
   labels = np.array (['eps', 'alpha', 'Dnu', 'numax', 'Hmax', 'Wenv', 'w', 'd02', 
                       'b02', 'd01', 'b01', 'd13', 'b03'])
 
-  orders = list_order_to_fit (numax, Dnu, n_order=n_order)
+  orders = list_order_to_fit (guess[3], guess[2], n_order=n_order)
 
   if not fit_l1 :
     fit_l3 = False
@@ -1057,7 +1168,7 @@ def perform_mle_pattern (Dnu, numax, Hmax, Wenv, Teff, freq, psd,
 
   cp_guess = np.reshape (guess, (1,guess.size))
   summary_guess = pd.DataFrame (data=cp_guess, columns=labels)
-  df_guess = pattern_to_a2z (*param, split=split, angle=angle, orders=orders)
+  df_guess = pattern_to_a2z (*param, split=split, angle=angle, orders=orders, amp_l=amp_l)
 
   # Transform width parameter to fit its logarithm
   param[6] = np.log (param[6])
@@ -1086,8 +1197,8 @@ def perform_mle_pattern (Dnu, numax, Hmax, Wenv, Teff, freq, psd,
   else :
     param_wdw = None
 
-  low_bound_freq = np.amin (df_guess.loc[df_guess[2]=='freq',4].to_numpy ()) - 0.2*Dnu
-  up_bound_freq = np.amax (df_guess.loc[df_guess[2]=='freq',4].to_numpy ()) + 0.2*Dnu
+  low_bound_freq = np.amin (df_guess.loc[df_guess[2]=='freq',4].to_numpy ()) - 0.2*guess[2]
+  up_bound_freq = np.amax (df_guess.loc[df_guess[2]=='freq',4].to_numpy ()) + 0.2*guess[2]
 
   aux_freq = freq[(freq>low_bound_freq)&(freq<up_bound_freq)]
   aux_psd = (psd/back)[(freq>low_bound_freq)&(freq<up_bound_freq)]
@@ -1098,7 +1209,7 @@ def perform_mle_pattern (Dnu, numax, Hmax, Wenv, Teff, freq, psd,
   with warnings.catch_warnings () :
     warnings.filterwarnings('ignore')
     result = minimize (log_likelihood_pattern, param, 
-              args=(aux_freq, aux_psd, aux_back, orders, split, angle, param_wdw),
+              args=(aux_freq, aux_psd, aux_back, orders, split, angle, param_wdw, amp_l),
               bounds=bounds, method=method, jac=jac, options=options, tol=tol)
 
   print (result.message)
@@ -1109,24 +1220,26 @@ def perform_mle_pattern (Dnu, numax, Hmax, Wenv, Teff, freq, psd,
 
   summary_result = pd.DataFrame (data=np.reshape (param_fitted, (1,len(labels))), columns=labels)
 
-  df_a2z = pattern_to_a2z (*param_fitted, split=split, angle=angle, orders=orders)
+  df_a2z = pattern_to_a2z (*param_fitted, split=split, angle=angle, orders=orders, amp_l=amp_l)
 
   visualise (param_fitted, aux_freq, aux_psd*aux_back, aux_back, param_pattern_2=guess, filename=filename,
-               orders=orders, split=split, angle=angle, snr=True, param_wdw=param_wdw, show=show) 
+               orders=orders, split=split, angle=angle, snr=True, param_wdw=param_wdw, show=show, amp_l=amp_l) 
 
   return df_a2z, param_fitted
 
 def explore_distribution_pattern (Dnu, numax, Hmax, Wenv, Teff, freq, 
                                   psd, back=None, wdw=None,
                                   n_order=3, split=0, angle=90, fit_l1=False, 
-                                  fit_l3=False, mass=None, guess=None, show=True,
+                                  fit_l3=False, mass=None, guess=None, 
+                                  low_bounds=None, up_bounds=None, show=True,
                                   show_corner=True, nsteps=1000, filename=None, 
                                   parallelise=False, progress=False, 
                                   nwalkers=64, filemcmc=None, coeff_discard=50, 
                                   thin=1, save_only_after_sampling=False,
                                   epsilon=None, width=None, alpha=None, d02=None,
                                   d01=None, d13=None,
-                                  bins=100, existing_chains='read') :
+                                  bins=100, existing_chains='read', 
+                                  amp_l=[1., 1.5, 0.7, 0.2]) :
                         
   '''
   Perform minimization of the log likelihood function according to the parameters set to vary.
@@ -1161,10 +1274,19 @@ def explore_distribution_pattern (Dnu, numax, Hmax, Wenv, Teff, freq,
   :type mass: float
 
   :param guess: MCMC will take it as first guess of param if given. Optional, default None.
-    Parameters must be given in the following order:
-    >>>> eps, alpha, Dnu, numax, Hmax, Wenv, w, d02, d01 (if fit_l1 is set to ``True``) <<<<
+    Parameters must be given in the following order: ``eps``, ``alpha``, ``Dnu``, ``numax``, 
+    ``Hmax``, ``Wenv, ``w``, ``d02``, ``b02``, ``d01``,
+    ``b01``, ``d13``, ``b03``.
   :type guess: array-like
 
+  :param low_bounds: lower bounds to consider in the parameter space exploration. Must have the same structure
+    than ``guess``.
+  :type low_bounds: ndarray
+
+  :param up_bounds: upper bounds to consider in the parameter space exploration. Must have the same structure
+    than ``guess``.
+  :type up_bounds: ndarray
+
   :param show: if set to ``True``, will plot a summary plot of guess (blue) and fitted (red) 
     pattern.
   :type show: bool
@@ -1250,6 +1372,11 @@ def explore_distribution_pattern (Dnu, numax, Hmax, Wenv, Teff, freq,
   else :
     xxx, bounds = first_guess_mcmc (Dnu, numax, Teff, Hmax, Wenv, mass=mass, return_bounds=True)
 
+  if low_bounds is not None :
+    bounds[:,0] = low_bounds
+  if up_bounds is not None :
+    bounds[:,1] = up_bounds
+
   if epsilon is not None :
     guess, bounds = override_epsilon (epsilon, guess, bounds, mcmc=True)
   if alpha is not None :
@@ -1267,7 +1394,7 @@ def explore_distribution_pattern (Dnu, numax, Hmax, Wenv, Teff, freq,
 
   labels = np.array (['eps', 'alpha', 'Dnu', 'numax', 'Hmax', 'Wenv', 'w', 'd02', 
                       'b02', 'd01', 'b01', 'd13', 'b03'])
-  orders = list_order_to_fit (numax, Dnu, n_order=n_order)
+  orders = list_order_to_fit (guess[3], guess[2], n_order=n_order)
 
   if not fit_l1 :
     fit_l3 = False
@@ -1288,7 +1415,7 @@ def explore_distribution_pattern (Dnu, numax, Hmax, Wenv, Teff, freq,
 
   cp_guess = np.reshape (guess, (1,guess.size))
   summary_guess = pd.DataFrame (data=cp_guess, columns=labels) 
-  df_guess = pattern_to_a2z (*param, split=split, angle=angle, orders=orders)
+  df_guess = pattern_to_a2z (*param, split=split, angle=angle, orders=orders, amp_l=amp_l)
 
   if back is None :
     back = np.full (psd.size, 1.)
@@ -1303,8 +1430,8 @@ def explore_distribution_pattern (Dnu, numax, Hmax, Wenv, Teff, freq,
   param[6] = np.log (param[6])
   bounds[6,:] = np.log (bounds[6,:])
 
-  low_bound_freq = np.amin (df_guess.loc[df_guess[2]=='freq',4].to_numpy ()) - 0.2*Dnu
-  up_bound_freq = np.amax (df_guess.loc[df_guess[2]=='freq',4].to_numpy ()) + 0.2*Dnu
+  low_bound_freq = np.amin (df_guess.loc[df_guess[2]=='freq',4].to_numpy ()) - 0.2*guess[2]
+  up_bound_freq = np.amax (df_guess.loc[df_guess[2]=='freq',4].to_numpy ()) + 0.2*guess[2]
 
   aux_freq = freq[(freq>low_bound_freq)&(freq<up_bound_freq)]
   aux_psd = (psd/back)[(freq>low_bound_freq)&(freq<up_bound_freq)]
@@ -1352,7 +1479,8 @@ def explore_distribution_pattern (Dnu, numax, Hmax, Wenv, Teff, freq,
 
     #saving parameters name and normalisation information
     filemcmc_info = filemcmc[:len(filemcmc)-3] + '.dat'
-    np.savetxt (filemcmc_info, np.c_[norm], fmt='%-s') 
+    np.savetxt (filemcmc_info, np.c_[norm, labels], fmt='%-s',
+                header=make_header_utility_file ()) 
 
     if save_only_after_sampling :
       # I have deliberately created the file to signal to other process that this chain is being
@@ -1365,7 +1493,7 @@ def explore_distribution_pattern (Dnu, numax, Hmax, Wenv, Teff, freq,
     print ('Beginning fit')
     sampler = emcee.EnsembleSampler(nwalkers, ndim, log_probability_pattern, 
                                     args=(aux_freq, aux_psd, bounds_to_pass, aux_back, orders, split, angle, param_wdw, 
-                                    norm), 
+                                    norm, amp_l), 
                                     backend=backend, pool=pool)
     sampler.run_mcmc(pos, nsteps, progress=progress)
 
@@ -1387,13 +1515,12 @@ def explore_distribution_pattern (Dnu, numax, Hmax, Wenv, Teff, freq,
   param_fitted = centiles[1,:]
   sigma_fitted = np.maximum (centiles[1,:] - centiles[0,:], centiles[2,:] - centiles[1,:])
 
-
   summary_result = pd.DataFrame (data=np.reshape (param_fitted, (1,len(labels))), columns=labels)
 
-  df_a2z = pattern_to_a2z (*param_fitted, split=split, angle=angle, orders=orders)
+  df_a2z = pattern_to_a2z (*param_fitted, split=split, angle=angle, orders=orders, amp_l=amp_l)
 
   visualise (param_fitted, aux_freq, aux_psd*aux_back, aux_back, param_pattern_2=guess, filename=filename,
-               orders=orders, split=split, angle=angle, snr=True, param_wdw=param_wdw, show=show) 
+             orders=orders, split=split, angle=angle, snr=True, param_wdw=param_wdw, show=show, amp_l=amp_l) 
 
   return df_a2z, param_fitted, sigma_fitted
 
