@@ -208,7 +208,6 @@ def peakbagging (a2z_file, freq, psd, back=None, wdw=None, dnu=None,
   :param back: activity background vector that will be used to complete the
     model to fit. Optional default ``None``.  Must have the same length than freq
     and psd. 
-
   :type back: ndarray.
 
   :param wdw: observation window (0 and 1 array of the same lenght as the
@@ -336,7 +335,7 @@ def peakbagging (a2z_file, freq, psd, back=None, wdw=None, dnu=None,
 
   :param fit_splittings: if the selected strategy is ``global``, the global
     ``split`` parameter will be fitted only if this parameter is set to ``True``.
-     Optional, default ``True``. 
+    Optional, default ``True``. 
   :type fit_splittings: bool
 
   :param fit_angle: if the selected strategy is ``global``, the global
@@ -428,12 +427,12 @@ def peakbagging (a2z_file, freq, psd, back=None, wdw=None, dnu=None,
     if order_to_fit is None :
       order_to_fit = order
 
-    print ('Orders to fit')
-    print (np.intersect1d (order, order_to_fit))
+    print ('Orders to fit:', np.array2string (np.intersect1d (order, order_to_fit),
+                                             separator=', ')[1:-1])
 
     for n in np.intersect1d (order, order_to_fit) :
+      print ('Fitting on order', n)
       if strategy=='order' :
-        print ('Fitting on order', n)
         df_a2z.loc[(df_a2z[0]==str(n))&(df_a2z[1]=='0'), 6] = 0
         df_a2z.loc[(df_a2z[0]==str(n))&(df_a2z[1]=='1'), 6] = 0
         df_a2z.loc[(df_a2z[0]==str(n-1))&(df_a2z[1]=='2'), 6] = 0
@@ -480,7 +479,6 @@ def peakbagging (a2z_file, freq, psd, back=None, wdw=None, dnu=None,
         df_a2z.loc[:,6] = 1
 
       if strategy=='pair' : 
-        print ('Fitting on order', n)
 
         if fit_02 :
 
