@@ -13,9 +13,11 @@ import matplotlib.ticker as ticker
 import matplotlib.pyplot as plt
 import emcee
 import corner
+import numba
 from apollinaire.psd import degrade_psd
 from .background import (background_model, extract_param,
                         log_prior, harvey)
+from .save_chain import save_sampled_chain
 
 
 def convert_prot (prot) :
@@ -26,146 +28,159 @@ def convert_prot (prot) :
 
   return f
 
-def create_label (n_harvey, n_harmonic) :
+def create_label (n_harmonic) :
+  '''
+  Create label of fitted parameters.
+  '''
 
   labels = ['f1']
   for ii in range (n_harmonic) :
     labels.append ('h_' + str (ii+1))
     labels.append ('w_' + str (ii+1))
-  for ii in range (n_harvey) :
-    labels.append ('A_H_' + str (ii+1))
-    labels.append ('nuc_H_' + str (ii+1))
-    labels.append ('alpha_H_' + str (ii+1))
+  labels.append ('noise')
 
   return labels
 
 
-def compute_cuts (f1, n_harmonic, numax) :
+def compute_cut (f1, n_harmonic) :
+  '''
+  Useful frequency cuts on the PSD.
+  '''
 
   cut_peak = (n_harmonic+2) * f1
-  high_cut = 1./3. * numax
 
-  return cut_peak, high_cut
+  return cut_peak
 
-def create_vector_to_fit (freq, psd, n_harmonic, low_cut, f1, numax, num=5000, smoothing=50) :
+def create_vector_to_fit (freq, psd, back, n_harmonic, low_cut, f1) :
+  '''
+  Compute background, power and frequency vectors that will be used for the fit. 
+  '''
 
-  cut_peak, high_cut = compute_cuts (f1, n_harmonic, numax)
+  cut_peak = compute_cut (f1, n_harmonic)
 
-  psd = psd[(freq>low_cut)&(freq<high_cut)]
-  freq = freq[(freq>low_cut)&(freq<high_cut)]
+  psd_fit = psd[(freq>low_cut)&(freq<cut_peak)]
+  freq_fit = freq[(freq>low_cut)&(freq<cut_peak)]
+  back_fit = back[(freq>low_cut)&(freq<cut_peak)]
 
-  aux_freq_1 = freq[freq<cut_peak]
-  aux_psd_1 = psd[freq<cut_peak]
-  aux_freq_2 = freq[freq>cut_peak]
-  aux_psd_2 = psd[freq>cut_peak]
+  return freq_fit, psd_fit, back_fit
 
-  aux_freq_2, aux_psd_2 = degrade_psd (aux_freq_2, aux_psd_2, smoothing=smoothing, num=num)
+def retype_vectors (freq, psd, back) :
 
-  freq_fit = np.concatenate ((aux_freq_1, aux_freq_2))
-  psd_fit = np.concatenate ((aux_psd_1, aux_psd_2))
+  freq = freq.astype (np.float64)
+  psd = psd.astype (np.float64)
+  back = back.astype (np.float64)
 
-  return psd_fit, freq_fit
+  return freq, psd, back
 
-def create_param_vector (f1, height, width, param_back, n_harvey=2, n_harmonic=1) :
+def create_param_vector (f1, log_height, log_width, n_harmonic=1) :
   '''
   Create input param vector for rotation peak fit.
 
   :param f1: initial guess for first harmonic frequency value.
   :type f1: float
 
-  :param height: height of the first harmonic
+  :param log_height: logarithm of height of the first harmonic
   :type height: float
  
-  :param width: width of the first harmonic
+  :param log_width: logarithm of width of the first harmonic
   :type width: float
 
-  :param_back: background parameter vector obtained with a background fit.
-  :type param_back: array-like
-
-  :param n_harvey: number of Harvey model which were used to fit the background.
-    Optional, default 2.
-  :type n_harvey: int
+  :return: input parameters
+  :rtype: ndarray
   '''
 
-  input_param = np.empty (1+2*n_harmonic+3*n_harvey)
+  input_param = np.empty (1+2*n_harmonic + 1, dtype=np.float_)
   
   input_param[0] = f1 
   for ii in range (n_harmonic) :
-    input_param [1+2*ii] = height / (ii+1)
-    input_param [2+2*ii] = width
-  input_param[1+2*n_harmonic:] = param_back[:3*n_harvey]
+    input_param [1+2*ii] = log_height - np.log (ii+1)
+    input_param [2+2*ii] = log_width
+  input_param[-1] = 1 #flat noise parameter
 
   return input_param
 
-def create_rotation_guess (freq, psd, prot, param_back, n_harvey=2, n_harmonic=1) :
+def create_rotation_guess (freq, psd, back, prot, n_harmonic=1) :
+  '''
+  Compute guess values for rotation fit.
+  '''
 
   f1 = convert_prot (prot)
-  back_model = background_model (freq, param_harvey=param_back[:3*n_harvey], n_harvey=n_harvey)
   w = 1e-2 * f1
-  aux = psd - back_model
-  h = np.maximum (0, np.mean (aux[(freq>f1-w)&(freq<f1+w)])) 
-  guess = create_param_vector (f1, h, w, param_back, n_harvey=n_harvey, n_harmonic=n_harmonic)
+  aux = psd / back
+  h = np.maximum (0, np.mean (aux[(freq>f1-w)&(freq<f1+w)]))  
+  log_h = np.log (h) 
+  log_w = np.log (w)
+  guess = create_param_vector (f1, log_h, log_w, n_harmonic=n_harmonic)
 
   return guess
 
 def get_low_bounds (guess, low_cut, n_harmonic=1) :
+  '''
+  Low bounds for prior on fitted values.
+  '''
 
   low_bounds = np.empty (guess.size)
   low_bounds[0] = 0.95 * guess[0]
-  low_bounds[1] = 0.1 * guess[1]
   for ii in range (n_harmonic) :
-    low_bounds[1+2*ii] = 0.
-    low_bounds[2+2*ii] = 0.
-  low_bounds[1+2*n_harmonic:] = 0.9 * guess[1+2*n_harmonic]
+    low_bounds[1+2*ii] = -50 #log
+    low_bounds[2+2*ii] = -50 #log
+  low_bounds[-1] = 0.
 
   return low_bounds
 
 def get_up_bounds (guess, n_harmonic=1) :
+  '''
+  Up bounds for prior on fitted values.
+  '''
 
   up_bounds = np.empty (guess.size)
   up_bounds [0] = 1.05 * guess[0]
-  up_bounds[1] = 3. * guess[1]
   for ii in range (n_harmonic) :
-    up_bounds[1+2*ii] = 5. * guess[1+2*ii]
-    up_bounds[2+2*ii] = 5. * guess[1+2*ii]
-  up_bounds[1+2*n_harmonic:] = 1.1 * guess[1+2*n_harmonic:]
+    up_bounds[1+2*ii] = np.log (50.) + guess[1+2*ii]
+    up_bounds[2+2*ii] = np.log (50.) + guess[2+2*ii]
+  up_bounds [-1] = 10
 
   return up_bounds
 
-def param_split (param, n_harmonic=1) :
+@numba.jit (nopython=True)
+def peak_model (freq, param_rot, n_harmonic, profile=1) :
+  '''
+  Peak model used to fit the rotation.
+  
+  :param profile: type of profile to use. 0: lorentzian, 1: sinc, 2: gaussian.
+  '''
 
-  param_rot = param[:1+2*n_harmonic]
-  param_harvey = param[1+2*n_harmonic:]
-
-  return param_harvey, param_rot
-
-def peak_model (freq, param_rot, n_harmonic, profile='gaussian') :
-
-  model = np.zeros (freq.size)
+  model = np.zeros (freq.size, dtype=np.float64)
 
   f1 = param_rot[0]
 
   for ii in range (n_harmonic) :
-    A = param_rot[1+2*ii] 
-    w = param_rot[2+2*ii] 
+    A = np.exp (param_rot[1+2*ii]) 
+    w = np.exp (param_rot[2+2*ii]) 
     xxx = (freq - (ii+1)*f1) / w
-    if profile == 'lorentzian' :
+    if profile == 0 :
       model += A / (1 + 4. * xxx * xxx)
-    if profile == 'sinc' :
+    if profile == 1 :
       model += A * np.sinc (xxx) * np.sinc (xxx)
-    if profile == 'gaussian' :
+    if profile == 2 :
       model += A * np.exp (- xxx * xxx)
 
   return model
 
-def rotation_model (freq, psd, param, n_harvey, n_harmonic, noise=0) :
+@numba.jit (nopython=True)
+def rotation_model (freq, param, n_harmonic, profile=1) :
+  '''
+  Wrapper for peak model. 
+  '''
+
+  param_rot = param[:-1]
+  model = peak_model (freq, param_rot, n_harmonic, profile=profile)
+  model += param[-1]
 
-  param_harvey, param_rot = param_split (param, n_harmonic)
-  model = background_model (freq, param_harvey, n_harvey=n_harvey, noise=noise)
-  model = model + peak_model (freq, param_rot, n_harmonic)
   return model
 
-def log_likelihood_rotation (param, freq, psd, n_harvey, n_harmonic, noise=0) :
+@numba.jit (nopython=True)
+def log_likelihood_rotation (param, freq, psd, back, n_harmonic, profile=1) :
   '''
   Compute negative log_likelihood for fitting model on 
   background.
@@ -180,23 +195,19 @@ def log_likelihood_rotation (param, freq, psd, n_harvey, n_harmonic, noise=0) :
   :param psd: power density vector in ppm^2/µHz or (m/s)^2/muHz.
   :type psd: ndarray
 
-  :param n_harvey: number of Harvey laws to use to build the background
-  model.  
-  :type n_harvey: int
-
   :param n_harmonic: number of Harvey laws to use to build the background
   model.  
   :type n_harmonic: int
   '''
 
-  model = rotation_model (freq, psd, param, n_harvey, n_harmonic, noise=noise)
+  model = rotation_model (freq, param, n_harmonic, profile=profile)
 
   aux = psd / model + np.log (model)
   log_l = np.sum (aux)
 
   return log_l
 
-def log_probability_rotation (param_to_fit, freq, psd, bounds, n_harvey, n_harmonic, norm=None, noise=0) :
+def log_probability_rotation (param_to_fit, freq, psd, back, bounds, n_harmonic, norm=None, profile=1) :
   '''
   Compute the positive posterior log probability (unnormalised) of the parameters to fit. 
 
@@ -211,10 +222,6 @@ def log_probability_rotation (param_to_fit, freq, psd, bounds, n_harvey, n_harmo
   :param psd: power density vector in ppm^2/µHz or (m/s)^2/muHz.
   :type psd: ndarray
 
-  :param n_harvey: number of Harvey laws to use to build the background
-  model.  
-  :type n_harvey: int
-
   :param n_harmonic: number of Harvey laws to use to build the background
   model.  
   :type n_harmonic: int
@@ -227,7 +234,8 @@ def log_probability_rotation (param_to_fit, freq, psd, bounds, n_harvey, n_harmo
   :rtype: float
   '''
 
-  param_to_fit = np.copy (param_to_fit) #make a copy to not modify the reference array
+  #param_to_fit = np.copy (param_to_fit) #make a copy to not modify the reference array
+  param_to_fit = param_to_fit.astype (np.float64) 
   bounds = np.copy (bounds)
 
   if norm is not None :
@@ -239,20 +247,21 @@ def log_probability_rotation (param_to_fit, freq, psd, bounds, n_harvey, n_harmo
 
   if not np.isfinite (l_prior) :
     return - np.inf
-  l_likelihood = - log_likelihood_rotation (param_to_fit, freq, psd, n_harvey, n_harmonic, noise=noise)
+  l_likelihood = - log_likelihood_rotation (param_to_fit, freq, psd, back, n_harmonic, profile)
 
   l_proba = l_prior + l_likelihood
 
   return l_proba
 
-def visualise_rotation (freq, psd, low_cut, f1, numax, param_fitted=None, guess=None, n_harvey=2,
-                        n_harmonic=3, filename=None, spectro=True, alpha=1., show=False, noise=0) :
+def visualise_rotation (freq, psd, back, low_cut, f1, param_fitted=None, guess=None, 
+                        n_harmonic=3, filename=None, spectro=True, alpha=1., show=False,
+                        profile=1) :
 
   '''
   Plot fitted rotation peaks against real PSD (and possibly against initial guess).
   '''
 
-  cut_peak, high_cut = compute_cuts (f1, n_harmonic, numax)
+  cut_peak = compute_cut (f1, n_harmonic)
 
   if (param_fitted is None) and (guess is None) :
     raise Exception ('No pattern was given to plot !')
@@ -260,34 +269,21 @@ def visualise_rotation (freq, psd, low_cut, f1, numax, param_fitted=None, guess=
 
   fig = plt.figure (figsize=(12,6))
   ax = fig.add_subplot (111)
+  
+  aux_freq = freq[(freq>low_cut)&(freq<cut_peak)]
 
-  ax.plot (freq, psd, color='darkgrey')
-  ax.plot (freq[(freq>low_cut)&(freq<high_cut)], psd[(freq>low_cut)&(freq<high_cut)], color='black')
+  ax.plot (freq[(freq>low_cut)&(freq<cut_peak)], psd[(freq>low_cut)&(freq<cut_peak)] / back[(freq>low_cut)&(freq<cut_peak)], color='black')
   if guess is not None :
-    guess_model = rotation_model (freq, psd, guess, n_harvey, n_harmonic, noise=noise)
-    ax.plot (freq, guess_model, color='green')
-    guess_harvey, param_rot = param_split (guess, n_harmonic)
-    guess_harvey = np.reshape (guess_harvey, (n_harvey, guess_harvey.size//n_harvey))
-    for elt in guess_harvey :
-      ax.plot (freq, harvey (freq, *elt), ':', color='green')
+    guess_model = rotation_model (aux_freq, guess, n_harmonic, profile=profile) 
+    ax.plot (aux_freq, guess_model, color='green')
   if param_fitted is not None :
-    fitted_model = rotation_model (freq, psd, param_fitted, n_harvey, n_harmonic, noise=noise)
-    ax.plot (freq, fitted_model, color='red', alpha=alpha)
-    fitted_harvey, param_rot = param_split (param_fitted, n_harmonic)
-    fitted_harvey = np.reshape (fitted_harvey, (n_harvey, fitted_harvey.size//n_harvey))
-    for elt in fitted_harvey :
-      ax.plot (freq, harvey (freq, *elt), '--', color='red', alpha=alpha)
+    fitted_model = rotation_model (aux_freq, param_fitted, n_harmonic, profile=profile) 
+    ax.plot (aux_freq, fitted_model, color='red', alpha=alpha)
 
   ax.set_xlabel (r'Frequency ($\mu$Hz)')
-  if spectro :
-    ax.set_ylabel (r'PSD ((m/s)$^2$/$\mu$Hz)')
-  else :
-    ax.set_ylabel (r'PSD (ppm$^2$/$\mu$Hz)')
+  ax.set_ylabel (r'PSD (SNR)')
 
   ax.set_xscale ('log')
-  ax.set_yscale ('log')
-
-  ax.set_ylim (0.9*np.amin(psd), 1.1*np.amax(psd))
 
   if filename is not None:
     plt.savefig (filename, format='pdf')
@@ -300,8 +296,8 @@ def visualise_rotation (freq, psd, low_cut, f1, numax, param_fitted=None, guess=
   return
 
 
-def perform_mle_rotation (freq, psd, param_back, prot, n_harvey=2, n_harmonic=3, low_cut=1., guess=None, low_bounds=None, up_bounds=None, method=_minimize_powell,
-                          filename=None, spectro=False, num=5000, smoothing=50, show=True, show_guess=False) :
+def perform_mle_rotation (freq, psd, back, prot, n_harmonic=3, low_cut=1., guess=None, low_bounds=None, up_bounds=None, method=_minimize_powell,
+                          filename=None, spectro=False, show=True, show_guess=False, profile=1) :
 
   '''
   Perform MLE over rotation model. 
@@ -312,13 +308,6 @@ def perform_mle_rotation (freq, psd, param_back, prot, n_harvey=2, n_harmonic=3,
   :param psd: power density vector in ppm^2/µHz or (m/s)^2/muHz.
   :type psd: ndarray
 
-  :param param_back: input parameters for the background.
-  :type param_back: array-like
-
-  :param n_harvey: number of Harvey laws to use to build the background
-    model. Optional, default 2. 
-  :type n_harvey: int
-
   :param n_harmonic: number of Harvey laws to use to build the background
     model. Optional, default 3.  
   :type n_harmonic: int
@@ -326,7 +315,7 @@ def perform_mle_rotation (freq, psd, param_back, prot, n_harvey=2, n_harmonic=3,
   :param guess: first guess directly passed by the users. If guess is ``None``, the 
     function will automatically infer a first guess. Optional, default ``None``.
     Parameters given in the following order:
-    *first harmonic frequency value, height, width, alpha, beta, param_harvey (3 n_harvey)*
+    *first harmonic frequency value, height, width, alpha, beta*
   :type guess: array-like.
 
   :param low_cut: Spectrum below this frequency will be ignored for the fit. The frequency value
@@ -356,24 +345,23 @@ def perform_mle_rotation (freq, psd, param_back, prot, n_harvey=2, n_harmonic=3,
     Optional, default ``False``.
   :type spectro: bool
 
-  :param num: number of points of the logarithmically sampled spectrum if ``quickfit`` is set to ``True``. Optional, default 50.
-  :type num: int
-
-  :param smoothing: size of the boxcar window used for the smoothing if set to ``True``. Optional, default 50.
-  :type smoothing: int
-
   :param show: if set to ``True``, will show at the end a plot summarising the fit. Optional, default ``True``.
   :type show: bool
 
   :param show_guess: if set to ``True``, will show at the beginning a plot summarising the guess. Optional, default ``False``.
   :type show_guess: bool
 
+  :param profile: model profile for rotation peak. 0:Lorentzian, 1:sinc, 2:Gaussian. Optional, default 1.
+  :type profile: int
+
   :return: fitted rotation model and corresponding parameters.
   :rtype: tuple of array
   '''
 
+  freq, psd, back = retype_vectors (freq, psd, back)
+
   if guess is None :
-    guess = create_rotation_guess (freq, psd, prot, param_back, n_harvey=n_harvey, n_harmonic=n_harmonic)
+    guess = create_rotation_guess (freq, psd, prot, n_harmonic=n_harmonic)
   if up_bounds is None :
     up_bounds = get_up_bounds (guess, n_harmonic=n_harmonic)
   if low_bounds is None :
@@ -383,38 +371,37 @@ def perform_mle_rotation (freq, psd, param_back, prot, n_harvey=2, n_harmonic=3,
   else :
     bounds = np.c_[low_bounds, up_bounds]
 
-  labels = create_label (n_harvey, n_harmonic)
-  numax = param_back[n_harvey*3+3]
-  noise = param_back[-1]
+  labels = create_label (n_harmonic)
 
-  aux_freq, aux_psd = create_vector_to_fit (freq, psd, n_harmonic, low_cut, guess[0], numax, num=num, smoothing=smoothing)
+  aux_freq, aux_psd, aux_back = create_vector_to_fit (freq, psd, back, n_harmonic, low_cut, guess[0])
+  aux_psd = aux_psd / aux_back
 
   if show_guess :
-    visualise_rotation (freq, psd, low_cut, f1, numax, param_fitted=None, guess=guess, n_harvey=n_harvey,
-                        n_harmonic=n_harmonic, filename=None, spectro=spectro, alpha=0.8, show=True, noise=noise)
+    visualise_rotation (freq, psd, back, low_cut, guess[0], param_fitted=None, guess=guess, profile=profile,
+                        n_harmonic=n_harmonic, filename=None, spectro=spectro, alpha=0.8, show=True)
 
   print ('Beginning fit')
   param = np.copy (guess)
   result = minimize (log_likelihood_rotation, param,
-                     args=(aux_freq, aux_psd, n_harvey, n_harmonic, noise), bounds=bounds,
+                     args=(aux_freq, aux_psd, aux_back, n_harmonic, profile), bounds=bounds,
                      method=method)
 
   print (result.message)
   param_model = result.x
 
-  fitted_model = rotation_model (freq, psd, param_model, n_harvey, n_harmonic, noise=noise)
+  fitted_model = rotation_model (freq, param_model, n_harmonic, profile=profile)
 
-  visualise_rotation (freq, psd, low_cut, guess[0], numax, param_fitted=param_model, guess=guess, n_harvey=n_harvey,
-                        n_harmonic=n_harmonic, filename=filename, spectro=spectro, alpha=0.8, show=show, noise=noise)
+  visualise_rotation (freq, psd, back, low_cut, guess[0], param_fitted=param_model, guess=guess, profile=profile,
+                        n_harmonic=n_harmonic, filename=filename, spectro=spectro, alpha=0.8, show=show)
 
   return fitted_model, param_model
 
 
-def explore_distribution_rotation (freq, psd, param_back, prot, n_harvey=2, n_harmonic=3, low_cut=1., guess=None, 
-                                   low_bounds=None, up_bounds=None, method=_minimize_powell,
-                                   filename=None, spectro=False, num=5000, smoothing=50, show=True, show_guess=False,
+def explore_distribution_rotation (freq, psd, back, prot, n_harmonic=3, low_cut=1., guess=None, 
+                                   low_bounds=None, up_bounds=None, 
+                                   filename=None, spectro=False, show=True, show_guess=False,
                                    show_corner=True, nsteps=1000, parallelise=False, progress=False, nwalkers=64, filemcmc=None,
-                                   coeff_discard=50, thin=1) :
+                                   coeff_discard=50, thin=1, save_only_after_sampling=True, profile=1) :
 
   '''
   Use a MCMC framework to fit the rotation model. 
@@ -425,13 +412,6 @@ def explore_distribution_rotation (freq, psd, param_back, prot, n_harvey=2, n_ha
   :param psd: power density vector in ppm^2/µHz or (m/s)^2/muHz.
   :type psd: ndarray
 
-  :param param_back: input parameters for the background.
-  :type param_back: array-like
-
-  :param n_harvey: number of Harvey laws to use to build the background
-    model. Optional, default 2. 
-  :type n_harvey: int
-
   :param n_harmonic: number of Harvey laws to use to build the background
     model. Optional, default 3.  
   :type n_harmonic: int
@@ -439,7 +419,7 @@ def explore_distribution_rotation (freq, psd, param_back, prot, n_harvey=2, n_ha
   :param guess: first guess directly passed by the users. If guess is ``None``, the 
     function will automatically infer a first guess. Optional, default ``None``.
     Parameters given in the following order:
-    *first harmonic frequency value, height, width, alpha, beta, param_harvey (3 n_harvey)*
+    *first harmonic frequency value, height, width, alpha, beta*
   :type guess: array-like.
 
   :param low_cut: Spectrum below this frequency will be ignored for the fit. The frequency value
@@ -466,12 +446,6 @@ def explore_distribution_rotation (freq, psd, param_back, prot, n_harvey=2, n_ha
     Optional, default ``False``.
   :type spectro: bool
 
-  :param num: number of points of the logarithmically sampled spectrum if ``quickfit`` is set to ``True``. Optional, default 50.
-  :type num: int
-
-  :param smoothing: size of the boxcar window used for the smoothing if set to ``True``. Optional, default 50.
-  :type smoothing: int
-
   :param show: if set to ``True``, will show at the end a plot summarising the fit. Optional, default ``True``.
   :type show: bool
 
@@ -503,26 +477,30 @@ def explore_distribution_rotation (freq, psd, param_back, prot, n_harvey=2, n_ha
   :param thin: take only every ``thin`` steps from the chain. Optional, default 1. 
   :type thin: int
 
+  :param profile: model profile for rotation peak. 0:Lorentzian, 1:sinc, 2:Gaussian. Optional, default 1.
+  :type profile: int
+
   :return: fitted rotation model, corresponding parameters and sigma obtained by MCMC exploration.
   :rtype: tuple of array
   '''
 
+  freq, psd, back = retype_vectors (freq, psd, back)
+
   if guess is None :
-    guess = create_rotation_guess (freq, psd, prot, param_back, n_harvey=n_harvey, n_harmonic=n_harmonic)
+    guess = create_rotation_guess (freq, psd, back, prot, n_harmonic=n_harmonic)
   if up_bounds is None :
     up_bounds = get_up_bounds (guess, n_harmonic=n_harmonic)
   if low_bounds is None :
     low_bounds = get_low_bounds (guess, low_cut, n_harmonic=n_harmonic)
 
-  labels = create_label (n_harvey, n_harmonic)
-  numax = param_back[n_harvey*3+3]
-  noise = param_back[-1]
+  labels = create_label (n_harmonic)
 
-  aux_freq, aux_psd = create_vector_to_fit (freq, psd, n_harmonic, low_cut, guess[0], numax, num=num, smoothing=smoothing)
+  aux_freq, aux_psd, aux_back = create_vector_to_fit (freq, psd, back, n_harmonic, low_cut, guess[0])
+  aux_psd = aux_psd / aux_back
 
   if show_guess :
-    visualise_rotation (freq, psd, low_cut, f1, numax, param_fitted=None, guess=guess, n_harvey=n_harvey,
-                        n_harmonic=n_harmonic, filename=None, spectro=spectro, alpha=0.8, show=True, noise=noise)
+    visualise_rotation (freq, psd, back, low_cut, guess[0], param_fitted=None, guess=guess, profile=profile, 
+                        n_harmonic=n_harmonic, filename=None, spectro=spectro, alpha=0.8, show=True)
 
   print ('Beginning fit')
 
@@ -549,6 +527,10 @@ def explore_distribution_rotation (freq, psd, param_back, prot, n_harvey=2, n_ha
   if filemcmc is not None :
     backend = emcee.backends.HDFBackend(filemcmc)
     backend.reset(nwalkers, ndim)
+    if save_only_after_sampling :
+      # I have deliberately created the file to signal to other process that this chain is being
+      # sampled
+      backend = None
     #saving parameters name and normalisation information
     filemcmc_info = filemcmc[:len(filemcmc)-3] + '.dat'
     np.savetxt (filemcmc_info, np.c_[norm], fmt='%-s')
@@ -556,11 +538,15 @@ def explore_distribution_rotation (freq, psd, param_back, prot, n_harvey=2, n_ha
     backend = None
 
   sampler = emcee.EnsembleSampler(nwalkers, ndim, log_probability_rotation,
-                                  args=(aux_freq, aux_psd, bounds_to_pass, n_harvey, n_harmonic,
-                                        norm, noise),
+                                  args=(aux_freq, aux_psd, aux_back, bounds_to_pass, n_harmonic,
+                                        norm, profile),
                                   backend=backend, pool=pool)
   sampler.run_mcmc(pos, nsteps, progress=progress)
 
+  if filemcmc is not None :
+    if save_only_after_sampling :
+      save_sampled_chain (filemcmc, sampler, ndim, nwalkers, nsteps)
+
   discard = nsteps // coeff_discard
 
   if show_corner :
@@ -572,10 +558,10 @@ def explore_distribution_rotation (freq, psd, param_back, prot, n_harvey=2, n_ha
   param_model = centiles[1,:]
   sigma_model = np.maximum (centiles[1,:] - centiles[0,:], centiles[2,:] - centiles[1,:])
 
-  fitted_model = rotation_model (freq, psd, param_model, n_harvey, n_harmonic, noise=noise)
+  fitted_model = rotation_model (freq, param_model, n_harmonic, profile=profile)
 
-  visualise_rotation (freq, psd, low_cut, guess[0], numax, param_fitted=param_model, guess=guess, n_harvey=n_harvey,
-                        n_harmonic=n_harmonic, filename=filename, spectro=spectro, alpha=0.8, show=show, noise=noise)
+  visualise_rotation (freq, psd, back, low_cut, guess[0], param_fitted=param_model, guess=guess, profile=profile, 
+                        n_harmonic=n_harmonic, filename=filename, spectro=spectro, alpha=0.8, show=show)
 
   return fitted_model, param_model, sigma_model
 
