@@ -17,14 +17,17 @@ import warnings
 def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
                        numax=None, Hmax=None, Wenv=None, epsilon=None, width=None, 
                        alpha=None, d02=None, d01=None, d13=None, n_harvey=2, low_cut=10.,
-                       guess_back=None, spectro=False, filename_back='background', back_mcmc=True,
+                       guess_back=None, low_bounds_back=None, up_bounds_back=None, 
+                       frozen_param_back=None, power_law=False, high_cut_plaw=20., spectro=False, 
+                       filename_back='background', back_mcmc=True, back_mle=True,
                        filemcmc_back='mcmc_sampler_background.h5', nsteps_mcmc_back=1000,
+                       guess_pattern=None, low_bounds_pattern=None, up_bounds_pattern=None,
                        pattern_mcmc=True, pattern_mle=True, n_order=3, n_order_peakbagging=3,
                        h0_screening=False, 
                        filename_pattern='pattern', filemcmc_pattern='mcmc_sampler_pattern.h5',
                        nsteps_mcmc_pattern=1000, parallelise=False, fit_l1=True, fit_l3=False,
                        use_numax_back=False,  progress=False, a2z_file='modes_param.a2z', a2z_in=None,
-                       nsteps_mcmc_peakbagging=1000, mcmcDir='.', instr='geometric',
+                       nsteps_mcmc_peakbagging=1000, mcmcDir='.', instr='geometric', amp_l=[1., 1.5, 0.7, 0.2],
                        filename_peakbagging='summary_peakbagging.pdf', nopeakbagging=False,
                        coeff_discard_back=5, coeff_discard_pattern=5, coeff_discard_pkb=5,
                        thin_back=1, thin_pattern=1, quickfit=False, num=500,
@@ -121,9 +124,31 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
     ``None``, the function will automatically infer a first guess. Optional,
     default ``None``.  Backgrounds parameters given in the following order:
     *param_harvey (3* ``n_harvey`` *), param_plaw (2), param_gaussian (3), white
-    noise (1)*. 
+    noise (1)*. You can use ``create_background_guess_arrays`` to create guess 
+    and bounds arrays with proper structure.  
   :type guess_back: array-like.
 
+  :param low_bounds_back: lower bounds to consider in the background parameters
+    space exploration. Must have the same dimension than ``guess_back``.
+  :type low_bounds_back: ndarray
+
+  :param up_bounds_back: upper bounds to consider in the background parameters
+    space exploration. Must have the same dimension than ``guess_back``.
+  :type up_bounds_back: ndarray
+
+  :param frozen_param_back: boolean array of the same size as ``guess_back``.
+    Components set to ``True`` will not be fitted.
+  :type frozen_param_back: boolean array
+
+  :param power_law: set to ``True`` to fit a power law on the background.
+    Optional, default ``False``.
+  :type power_law: bool
+
+  :param high_cut_plaw: if ``power_law=True``, the function will consider
+    the PSD segment between ``low_cut`` and ``high_cut_plaw`` to 
+    compute the power law parameters initial guess.
+  :type high_cut_plaw: float
+
   :param spectro: if set to ``True``, make the plot with unit consistent with
     radial velocity, else with photometry. Optional, default ``False``.
   :type spectro: bool
@@ -136,6 +161,10 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
   :param back_mcmc: If set to ``False``, no MCMC sampling will be performed 
     for the background. Optional, default ``True``.
   :type back_mcmc: bool
+
+  :param back_mle: If set to ``False``, no MLE optimisation 
+    will be performed for the background. Optional, default ``True``.
+  :type back_mle: bool
   
   :param filemcmc_back: Name of the hdf5 file where the MCMC background chain will 
     be stored. Optional, default ``mcmc_sampler_background.h5``.
@@ -145,6 +174,22 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
     background parameters exploration. Optional, default 1000.
   :type nsteps_mcmc_back: int
 
+  :param guess_pattern: first guess directly passed by the user. If guess is
+    ``None``, the function will automatically infer a first guess. Optional,
+    default ``None``.  Pattern parameters have to be given in the following order:
+    ``eps``, ``alpha``, ``Dnu``, ``numax``,  ``Hmax``, ``Wenv``, ``w``, ``d02``, 
+    ``b02``, ``d01``, ``b01``, ``d13``, ``b03``. A first version of the guess
+    and bounds array can be created through the ``create_pattern_guess_arrays``.
+  :type guess_pattern: array-like.
+
+  :param low_bounds_pattern: lower bounds to consider in the background parameters
+    space exploration. Must have the same dimension than ``guess_pattern``.
+  :type low_bounds_pattern: ndarray
+
+  :param up_bounds_pattern: upper bounds to consider in the background parameters
+    space exploration. Must have the same dimension than ``guess_pattern``.
+  :type up_bounds_pattern: ndarray
+
   :param pattern_mle: If set to ``False``, no MLE optimisation 
     will be performed for the pattern. Optional, default ``True``.
   :type pattern_mle: bool
@@ -224,6 +269,10 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
   :param instr: name of the instrument for which m-amplitude ratios will 
     be computed. ``geometric``, ``golf`` and ``virgo`` are available.
   :type instr: str
+
+  :param amp_l: amplitude ratio between harmonic degrees (relative to l=0 mode).
+    Optional, default [1., 1.5, 0.7, 0.2].
+  :type amp_l: array-like
   
   :param filename_peakbagging: name of the file where the plot summary 
     of the individual mode parameters peakbagging will be stored.
@@ -374,6 +423,9 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
   :return: ``None``
   '''
 
+  if back_mle==False and back_mcmc==False :
+    raise Exception ('You must set at least pattern_mcmc=True or pattern_mle=True')
+
   if pattern_mle==False and pattern_mcmc==False :
     raise Exception ('You must set at least pattern_mcmc=True or pattern_mle=True')
 
@@ -387,21 +439,25 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
                   Warning)
 
   if back is None :
-    fitted_back, param_back = perform_mle_background (freq, psd, n_harvey=n_harvey, r=r, m=m, teff=teff, guess=guess_back, 
-                              dnu=dnu, numax=numax, frozen_param=None, power_law=False,
-                              frozen_harvey_exponent=frozen_harvey_exponent, low_cut=low_cut, fit_log=True, quickfit=quickfit,
-                              low_bounds=None, up_bounds=None, no_bounds=False, show=False, show_guess=False, filename=filename_back+'.pdf', spectro=spectro,
-                              apodisation=apodisation, num=num, reboxing_behaviour=reboxing_behaviour, reboxing_strategy=reboxing_strategy)
-    np.savetxt (filename_back+'_parameters.dat', param_back, fmt='%-s', header=make_header_background_param (n_harvey=n_harvey, author=author, mcmc=False))
+    if back_mle :
+      fitted_back, param_back = perform_mle_background (freq, psd, n_harvey=n_harvey, r=r, m=m, teff=teff, guess=guess_back, 
+                                dnu=dnu, numax=numax, frozen_param=frozen_param_back, power_law=power_law,
+                                frozen_harvey_exponent=frozen_harvey_exponent, low_cut=low_cut, fit_log=True, quickfit=quickfit,
+                                low_bounds=low_bounds_back, up_bounds=up_bounds_back, no_bounds=False, high_cut_plaw=high_cut_plaw,
+                                show=False, show_guess=False, filename=filename_back+'.pdf', spectro=spectro,
+                                apodisation=apodisation, num=num, reboxing_behaviour=reboxing_behaviour, reboxing_strategy=reboxing_strategy)
+      np.savetxt (filename_back+'_parameters.dat', param_back, fmt='%-s', header=make_header_background_param (n_harvey=n_harvey, author=author, mcmc=False))
+    else :
+      param_back = guess_back
 
     if back_mcmc :
-      fitted_back, param_back, sigma_back  = explore_distribution_background (freq, psd,
-                                       n_harvey=n_harvey, r=r, m=m, teff=teff, guess=param_back, frozen_param=None, power_law=False,
+      fitted_back, param_back, sigma_back = explore_distribution_background (freq, psd,
+                                       n_harvey=n_harvey, r=r, m=m, teff=teff, guess=param_back, frozen_param=frozen_param_back, power_law=power_law,
                                        frozen_harvey_exponent=frozen_harvey_exponent, low_cut=low_cut, fit_log=True, quickfit=quickfit,
-                                       low_bounds=None, up_bounds=None, spectro=spectro, show=False, show_guess=False, show_corner=True,
+                                       low_bounds=low_bounds_back, up_bounds=up_bounds_back, spectro=spectro, show=False, show_guess=False, show_corner=True,
                                        nsteps=nsteps_mcmc_back, filename=filename_back+'.pdf', parallelise=progress, progress=progress, nwalkers=nwalkers, 
                                        filemcmc=filemcmc_back, coeff_discard=coeff_discard_back, thin=thin_back, 
-                                       save_only_after_sampling=save_only_after_sampling,
+                                       save_only_after_sampling=save_only_after_sampling, high_cut_plaw=high_cut_plaw,
                                        apodisation=apodisation, num=num, reboxing_behaviour=reboxing_behaviour,
                                        reboxing_strategy=reboxing_strategy, bins=bins, existing_chains=existing_chains_back)
       np.savetxt (filename_back+'_parameters.dat', np.c_[param_back, sigma_back], fmt='%-s', 
@@ -437,7 +493,8 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
     if pattern_mle :
       df_a2z, pattern = perform_mle_pattern (dnu, numax, Hmax, Wenv, teff, freq, psd, back=fitted_back, 
                                              wdw=None, epsilon=epsilon, width=width, alpha=alpha, d01=d01,
-                                             d02=d02, d13=d13,
+                                             d02=d02, d13=d13, guess=guess_pattern, low_bounds=low_bounds_pattern,
+                                             up_bounds=up_bounds_pattern, amp_l=amp_l,
                                              n_order=n_order, split=0, angle=90, fit_l1=fit_l1, 
                                              fit_l3=fit_l3, mass=m, show=False, filename=filename_pattern+'.pdf')
       # Setting value to None to avoid overriding the MLE results at the MCMC step
@@ -449,19 +506,20 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
       width = None
       
     else :
-      pattern = None
+      pattern = guess_pattern
 
     if pattern_mcmc :
       df_a2z, pattern, sigma_pattern = explore_distribution_pattern (dnu, numax, Hmax, Wenv, teff, freq, psd, back=fitted_back, wdw=None,
                                                                      n_order=n_order, split=0, angle=90, fit_l1=fit_l1, 
-                                                                     fit_l3=fit_l3, mass=m, guess=pattern, show=False,
+                                                                     fit_l3=fit_l3, mass=m, guess=pattern, low_bounds=low_bounds_pattern,
+                                                                     up_bounds=up_bounds_pattern, show=False,
                                                                      show_corner=True, nsteps=nsteps_mcmc_pattern, 
                                                                      filename=filename_pattern+'.pdf', parallelise=parallelise, progress=progress,
                                                                      nwalkers=nwalkers, filemcmc=filemcmc_pattern, coeff_discard=coeff_discard_pattern, 
                                                                      thin=thin_pattern, save_only_after_sampling=save_only_after_sampling, 
                                                                      bins=bins, existing_chains=existing_chains_pattern,
                                                                      epsilon=epsilon, alpha=alpha, width=width,
-                                                                     d02=d02, d01=d01, d13=d13)
+                                                                     d02=d02, d01=d01, d13=d13, amp_l=amp_l)
 
       np.savetxt (filename_pattern+'.dat', np.c_[pattern, sigma_pattern], fmt='%-s', 
                   header=make_header_pattern (author=author, fit_l1=fit_l1, fit_l3=fit_l3))
@@ -481,7 +539,7 @@ def stellar_framework (freq, psd, r, m, teff, nwalkers=64, dnu=None, back=None,
       fit_angle = False
       fit_splittings = False
     df_a2z = pattern_to_a2z (*pattern, split=guess_split, angle=guess_angle, orders=orders_for_peakbagging, angle_order=fit_angle, 
-                             splitting_order=fit_splittings, common_width=common_width)
+                             splitting_order=fit_splittings, common_width=common_width, amp_l=amp_l)
     if h0_screening :
       #Max rebinning is at 5 muHz (arbitrary) or 99 bin
       tmax = min (5 / np.median (np.diff (freq)), 99)
