@@ -29,14 +29,28 @@ def test_a2z (a2z_file) :
 
   state = True
 
-  if np.any (np.isnan (pkb)) :
-    state = False 
+  assert ~np.any (np.isnan (pkb)), 'The pkb array contains NaN.' 
+
+  return df_a2z, pkb
 
-  return state
 
 if __name__ == '__main__' :
 
   f = importlib.resources.path (templates, 'test.a2z')
   with f as filename :
-    print (test_a2z (filename))
+    df_a2z, pkb = test_a2z (filename)
+  assert np.all (get_list_order (df_a2z)==[5, 21, 22]), 'The list of order read from the a2z DataFrame is not correct'
+  f = importlib.resources.path (templates, 'verif.pkb')
+  with f as filename :
+    verif_pkb = np.loadtxt (filename)
+  residual = np.abs (pkb - verif_pkb)
+  error = np.linalg.norm (residual.ravel(), ord=np.inf)
+  assert error < 1.e-6, 'The pkb array does not contain the expected values.'
+  freq = np.linspace (0, 5000, 10000)
+  model = compute_model (freq, pkb)
+  assert ~np.any (np.isnan (model)), 'The model built from the test pkb array contains NaN.'
+
+
+
+
 
