@@ -2,8 +2,10 @@ import numpy as np
 from astropy.io import fits
 from astropy.table import Table
 import matplotlib
+from matplotlib import colors
 import matplotlib.pyplot as plt
 import pandas as pd
+from scipy.interpolate import interp1d
 
 '''
 Module to compute power spectra density (PSD) from
@@ -93,8 +95,10 @@ def psd_to_fits (freq, psd, filename) :
 
   :param freq: frequency vector.
   :type freq: ndarray
+
   :param psd: psd vector.
   :type psd: ndarray
+
   :param filename: name of the fits file to create.
   :type filename: str
   '''
@@ -105,6 +109,40 @@ def psd_to_fits (freq, psd, filename) :
 
   return
 
+def psd_second (freq, psd, resampling_length=1000) :
+  '''
+  Compute PS2, that is the PSD of a given PSD. The returned
+  power spectrum PS2 is normalised by its standard deviation.
+
+  :param freq: frequency vector.
+  :type freq: ndarray
+
+  :param psd: psd vector.
+  :type psd: ndarray
+
+  :return: period and second PSD vector 
+  :rtype: tuple of arrays
+  '''
+
+  period = 1 / freq[freq>0]
+  func = interp1d (period, psd[freq>0])
+  resampled_period = np.linspace (period[0], period[-1], resampling_length)
+  resampled_psd = func (resampled_period)
+  resampled_period = np.flip (resampled_period)
+  resampled_psd = np.flip (resampled_psd)
+  d = resampled_period[1] - resampled_period[0]
+
+  dft = 2 *  np.fft.rfft (resampled_psd) / resampled_psd.size 
+  ps2 = np.abs (dft) [1:]
+  freq2 = np.fft.rfftfreq (resampled_psd.size, d=d)
+  period2 = 1 / freq2[1:]
+  ps2 = np.flip (ps2)
+  period2 = np.flip (period2)
+  ps2 = ps2 / np.std (ps2)
+
+  return period2, ps2
+
+
 def plot_psd_fits (filename, rv=True, index=0) :
   '''
   Plot the PSD of a fits file containing frequency and power 
@@ -138,7 +176,8 @@ def plot_psd_fits (filename, rv=True, index=0) :
 
 def echelle_diagram (freq, PSD, dnu, twice=False, fig=None, index=111,
                      figsize=(16,16), title=None,
-                     smooth=10, cmap='cividis', mode_freq=None, mode_freq_err=None,
+                     smooth=10, cmap='cividis', cmap_scale='linear', 
+                     mode_freq=None, mode_freq_err=None,
                      vmin=None, vmax=None, scatter_color='white', fmt='+', ylim=None,
                      shading='auto', **kwargs) :
 
@@ -177,6 +216,10 @@ def echelle_diagram (freq, PSD, dnu, twice=False, fig=None, index=111,
   :param cmap: select one available color map provided by matplotlib, default ``cividis``
   :type cmap: str
 
+  :param cmap_scale: scale use for the colormap. Can be 'linear' or 'logarithmic'.
+    Optional, default 'linear'.
+  :type cmap_scale: str
+
   :param mode_freq: frequency array of the modes to represent on the diagram. It can be single 
     array or a tuple of array.
   :type mode_freq: ndarray or tuple of array
@@ -214,6 +257,13 @@ def echelle_diagram (freq, PSD, dnu, twice=False, fig=None, index=111,
   :rtype: matplotlib Figure
   '''
 
+  if cmap_scale not in ['linear', 'logarithmic'] :
+    raise Exception ("cmap_scale should be set to 'linear' or 'logarithmic'.")
+
+  if cmap_scale=='logarithmic':
+    norm = colors.LogNorm (vmin=vmin, vmax=vmax)
+  elif cmap_scale=='linear' :
+    norm = colors.Normalize (vmin=vmin, vmax=vmax)
 
   if smooth != 1 :
     PSD = pd.Series(data=PSD).rolling (window=smooth, min_periods=1, 
@@ -241,7 +291,8 @@ def echelle_diagram (freq, PSD, dnu, twice=False, fig=None, index=111,
   if fig is None : 
     fig = plt.figure (figsize=figsize)
   ax = fig.add_subplot (index)
-  ax.pcolormesh (x_freq, y_freq, ed, cmap=cmap, vmin=vmin, vmax=vmax, shading=shading)
+  ax.pcolormesh (x_freq, y_freq, ed, cmap=cmap,  
+                 norm=norm, shading=shading)
 
   if mode_freq is not None :
     if type (mode_freq) is not tuple :
