@@ -10,13 +10,16 @@ import h5py
 import sys
 from .songlib import *
 from .interval_nan import flux_nan, manual_nan
+from .correction_methods import *
+from apollinaire.processing import design_digital_filter
 
 '''
 Standard correction methods for Solar-SONG data.
 '''
 
 def standard_correction (dir_velocity, dir_corrected, dir_flux_out=None,
-                         dir_long=None,  dir_flux=None, iSONG=False, dt=20., pop_end=0, pop_beginning=0, save=False,
+                         dir_long=None,  dir_flux=None, clean_directories=False, 
+                         iSONG=False, dt=20., pop_end=0, pop_beginning=0, save=False,
                          correct=True, test=False, test_length=2, weighted=True, perf_lowess=False,
                          remove_interval_manually=True, plot_flux=False, 
                          cut_rising=12000, sigma_box=[3.,2.5], size_neighborhood=50, sigma_neighborhood=[3.5,3.],
@@ -47,6 +50,10 @@ def standard_correction (dir_velocity, dir_corrected, dir_flux_out=None,
     default ``None``. 
   :type dir_flux_out: string
 
+  :param clean_directories: if set to ``True``, will clean the output directories before reducing any data.
+    Optional, default ``False``.
+  :type clean_directories: bool
+
   :param iSONG: set to ``True`` if the data to use are outputs (with chunks already integrated) from the iSONG pipeline.
     Optional, default ``False``.
   :type iSONG: bool
@@ -165,6 +172,17 @@ def standard_correction (dir_velocity, dir_corrected, dir_flux_out=None,
   :rtype: tuple of ndarray
   '''
 
+  if clean_directories :
+    print ('Cleaning output directories')
+    list_dir_to_clean = [dir_corrected, dir_flux_out]
+    for dir_to_clean in list_dir_to_clean :
+      if dir_to_clean is not None :
+        for f in glob.glob (path.join (dir_to_clean, '*.h5')) :
+          os.remove (f)
+    if dir_long is not None :
+      os.remove (path.join (dir_long, 'solar_song_long_corrected.h5'))
+      os.remove (path.join (dir_long, 'solar_song_long_corrected.dat'))
+
   dir_src = os.path.abspath ('.') 
   
   day = int (86400 / dt)
@@ -183,26 +201,25 @@ def standard_correction (dir_velocity, dir_corrected, dir_flux_out=None,
     filt = design_digital_filter (numtaps=numtaps, bands=bands, desired=desired,
                                   plot_response=plot_response, fs=1/dt)
   
-    os.chdir (dir_velocity)
     if iSONG==True :
-      list_velocity = glob.glob ('*.cln')
+      list_velocity = glob.glob (path.join (dir_velocity, '*.cln'))
     else :
-      list_velocity = glob.glob ('*.h5')
-      list_flux = glob.glob ('*.h5')
+      list_velocity = glob.glob (path.join (dir_velocity, '*.h5'))
+      list_flux = glob.glob (path.join (dir_flux, '*.h5'))
+
+    list_velocity.sort ()
     for ii in range (pop_end) : 
       list_velocity.pop ()
     for ii in range (pop_beginning) : 
       list_velocity.pop (0)
 
     if iSONG==False : 
-      os.chdir (dir_flux)
+      list_flux.sort ()
       for ii in range (pop_end) :
         list_flux.pop ()
       for ii in range (pop_beginning) : 
         list_flux.pop (0)
   
-    os.chdir (dir_src)
-  
     if test==True :
       list_velocity = list_velocity[0:test_length]
       if iSONG==False :
@@ -214,7 +231,7 @@ def standard_correction (dir_velocity, dir_corrected, dir_flux_out=None,
     if iSONG==True :
       for file_v in list_velocity :
         print (file_v)
-        file_id = file_v[:13]
+        file_id = path.basename (file_v)[:13]
         fileout = path.join (dir_corrected, file_id + '_corrected.h5')
         # usecols=(1,4,7) for BVC velocities - (1,3,7) otherwise
         # The ephemeris files seems to already have barycentric correction
@@ -239,7 +256,7 @@ def standard_correction (dir_velocity, dir_corrected, dir_flux_out=None,
     else :
       for file_v, file_f in zip (list_velocity, list_flux) :
         print (file_v, file_f)
-        file_id = file_v[:9]
+        file_id = path.basename (file_v)[:9]
         fileout = path.join (dir_corrected, file_id + '_corrected.h5')
         if plot_flux == True :
           fileout_flux = path.join (dir_flux_out, file_id + '_corrected.h5')
@@ -254,17 +271,11 @@ def standard_correction (dir_velocity, dir_corrected, dir_flux_out=None,
           weights = compute_weights (mv, cut_up=cut_up, cut_low=cut_low)
         else :
           weights = None
-  
-        # ------------------------
-        #TEST : FLUX THRESHOLD
-        weights[np.nanmean(mf, axis=0) < cut_flux] = 0
-        # ------------------------
         
         mean_flux = np.nanmean (mf, axis=0)
         v = integrate_chunk (mv, weights=weights) 
         f = integrate_chunk (mf, weights=weights)
   
-        #rs = rebox_stamps (s, dt=dt)
         aux_s, rf = rebox_array (s, f, dt=dt, velocity=False)
         rs, v, mask_interpo = daily_correction_1d (s, v, f, plot=plot_intermediate, dt=dt, rebox=True, 
                                      sigma_box=sigma_box, thresh=thresh, perf_lowess=perf_lowess,  
@@ -282,17 +293,13 @@ def standard_correction (dir_velocity, dir_corrected, dir_flux_out=None,
   # CONCATENING DAILY CORRECTED SERIES 
   # FINAL CORRECTIONS AND PLOT
   # ------------------------------------------------------------------------------------------------
-  os.chdir (dir_corrected) 
-  list_corrected = glob.glob ('*.h5')
+  list_corrected = glob.glob (path.join (dir_corrected, '*.h5'))
   s, v = concatene_days (list_corrected, chunk=False, dt=dt)
   
   if plot_flux==True :
-    os.chdir (dir_flux_out) 
-    list_flux_corrected = glob.glob ('*.h5')
+    list_flux_corrected = glob.glob (path.join (dir_flux_out, '*.h5'))
     xxx, f = concatene_days (list_flux_corrected, chunk=False, dt=dt, key='fluxlevel')
   
-  os.chdir (dir_src)
-  
   # flux correction (already done in previous steps)
   v = flux_nan (s, v)
   v[np.isnan(v)]=0.
