@@ -12,12 +12,22 @@
 #
 import os
 import sys
+from pkg_resources import DistributionNotFound, get_distribution
+
+try:
+    __version__ = get_distribution("apollinaire").version
+except DistributionNotFound:
+    __version__ = "unknown version"
+
+
 sys.path.insert(0, os.path.abspath('../../apollinaire'))
  
-autodoc_mock_imports = ['numpy', 'scipy', 'matplotlib', 'matplotlib.pyplot', 'scipy.optimize', 'matplotlib.ticker', 'corner', 'emcee',
-                       'pandas', 'astropy', 'numdifftools', 'george', 'tqdm', 'dill', 'pathos', 'pathos.multiprocessing', 'h5py',
-                       'statsmodels.nonparametric.smoothers_lowess', 'pycorrelate', 'numba', 'scipy.io', 'statsmodels',
-                       'np', 'pd', 'plt', 'scipy.interpolate', 'astropy.stats', 'scipy.signal', 'joblib']
+autodoc_mock_imports = ['numpy', 'scipy', 'matplotlib', 'matplotlib.pyplot',
+                        'scipy.optimize', 'matplotlib.ticker', 'corner', 'emcee', 'pandas', 'astropy',
+                        'numdifftools', 'george', 'tqdm', 'dill', 'pathos', 'pathos.multiprocessing',
+                        'h5py', 'statsmodels.nonparametric.smoothers_lowess', 'pycorrelate', 'numba',
+                        'scipy.io', 'statsmodels', 'np', 'pd', 'plt', 'scipy.interpolate',
+                        'astropy.stats', 'scipy.signal', 'joblib']
 
 # -- Project information -----------------------------------------------------
 
@@ -25,9 +35,11 @@ project = 'apollinaire'
 copyright = '2020, Sylvain N. Breton'
 author = 'Sylvain N. Breton'
 master_doc = 'index'
+source_suffix = ".rst"
 
 # The full version, including alpha/beta/rc tags
-release = '0.15'
+release = __version__
+version = __version__
 
 
 # -- General configuration ---------------------------------------------------
@@ -35,11 +47,11 @@ release = '0.15'
 # Add any Sphinx extension module names here, as strings. They can be
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
 # ones.
-extensions = ['sphinx.ext.autodoc', 'sphinx_rtd_theme', 'sphinx.ext.napoleon',
-              'sphinx.ext.mathjax']
-
-# Add any paths that contain templates here, relative to this directory.
-templates_path = ['_templates']
+extensions = ['sphinx.ext.autodoc', 
+              'sphinx.ext.napoleon',
+              'sphinx.ext.mathjax', 
+              'sphinx_book_theme', 
+              'IPython.sphinxext.ipython_console_highlighting']
 
 # List of patterns, relative to source directory, that match files and
 # directories to ignore when looking for source files.
@@ -49,15 +61,20 @@ exclude_patterns = ['notebooks/golf_timing']
 
 # -- Options for HTML output -------------------------------------------------
 
-# The theme to use for HTML and HTML Help pages.  See the documentation for
-# a list of builtin themes.
-#
-#html_theme = 'alabaster'
-html_theme = 'sphinx_rtd_theme'
-
-# Add any paths that contain custom static files (such as style sheets) here,
-# relative to this directory. They are copied after the builtin static files,
-# so a file named "default.css" will overwrite the builtin "default.css".
+html_theme = 'sphinx_book_theme'
 html_static_path = ['_static']
-
+html_title = "apollinaire"
+html_theme_options = {
+     "path_to_docs": "docs",
+     "repository_url": "https://gitlab.com/sybreton/apollinaire/",
+     "use_repository_button": True,
+     "use_download_button": True,
+     }
+html_sidebars = {
+    "**": [
+     "search-field.html", 
+     "sbt-sidebar-nav.html", 
+     "sbt-sidebar-footer.html"
+          ]
+     }
 numpydoc_show_class_members = False
